"""empty message

Revision ID: roc_dingo_0020_rm_biacurrtablehfr
Revises: roc_dingo_0019_modify_views_per_date
Create Date: 2021-12-17 10:13:35.507837

"""

import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

from poppy.pop.alembic.helpers import create_table, execute, user, drop_table

# revision identifiers, used by Alembic.
revision = "roc_dingo_0020_rm_biacurrtablehfr"
down_revision = "roc_dingo_0019_modify_views_per_date"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    drop_table("bia_current_log", schema="pipeline")
    drop_table("bia_sweep_table", schema="pipeline")
    drop_table("hfr_freq_list", schema="pipeline")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    create_table(
        "bia_sweep_table",
        sa.Column("id", sa.BIGINT(), nullable=False),
        sa.Column("tc_name", sa.String(length=256), nullable=False),
        sa.Column("utc_time", postgresql.TIMESTAMP(), nullable=False),
        sa.Column("sweep_eeprom", sa.BOOLEAN(), nullable=True),
        sa.Column("sweep_step_cur", sa.String(), nullable=True),
        sa.Column("insert_time", postgresql.TIMESTAMP(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("utc_time", "tc_name"),
        schema="pipeline",
    )
    create_table(
        "hfr_freq_list",
        sa.Column("id", sa.BIGINT(), nullable=False),
        sa.Column("utc_time", postgresql.TIMESTAMP(), nullable=False),
        sa.Column("tc_name", sa.String(length=256), nullable=False),
        sa.Column("freq_list", sa.String(), nullable=False),
        sa.Column("freq_nr", sa.INTEGER(), nullable=False),
        sa.Column("insert_time", postgresql.TIMESTAMP(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("utc_time", "tc_name"),
        schema="pipeline",
    )
    create_table(
        "bia_current_log",
        sa.Column("id", sa.BIGINT(), nullable=False),
        sa.Column("tc_name", sa.String(length=256), nullable=False),
        sa.Column("utc_time", postgresql.TIMESTAMP(), nullable=False),
        sa.Column("bia_current", sa.FLOAT(), nullable=False),
        sa.Column("insert_time", postgresql.TIMESTAMP(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("utc_time", "tc_name"),
        schema="pipeline",
    )

    # added by hand
    # fix table permissions
    grant_user_seq = """GRANT USAGE, SELECT
        ON ALL SEQUENCES IN SCHEMA pipeline TO {0}""".format(user)
    execute(grant_user_seq)
    # ### end Alembic commands ###



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Contains dingo tasks to synchronize the file tree with the ROC database."""

import urllib.request
import urllib.parse
from urllib.error import HTTPError
import json

from datetime import timedelta

from sqlalchemy import and_
from sqlalchemy.sql import func
from sqlalchemy.dialects import postgresql
from sqlalchemy.orm import load_only

from poppy.core.logger import logger
from poppy.core.db.connector import Connector
from poppy.core.task import Task
from poppy.core.target import BaseTarget

from roc.dingo.constants import PIPELINE_DATABASE
from roc.dingo.models.file import FileLog


__all__ = ["CheckSOARDataAvailability"]


class CheckSOARDataAvailability(Task):
    """
    For each files flagged as delivered in DB, search SOAR if it available
    if yes : set is_archived
    """

    plugin_name = "roc.dingo"
    name = "check_soar"

    def add_targets(self):
        logger.debug("CheckSOARDataAvailability() : add_targets")

        # number of archived files
        self.add_output(target_class=BaseTarget, many=True, identifier="store_archived")

    def setup_inputs(self):
        """
        Setup task inputs.

        :param task:
        :return:
        """

        logger.debug("CheckSOARDataAvailability() : setup_inputs")

        # get the TAP URL
        self.soar_tap_url = self.pipeline.get("url", args=True)

    def run(self):
        logger.debug("CheckSOARDataAvailability() : run")

        # Initialize inputs
        self.setup_inputs()

        # Initialize counters
        cpt_archived = 0  # files really stored as is_archived
        cpt_to_archive = 0  # files to be stored as is_archived
        cpt_files_to_archive = 0  # files delivered but not archived yet
        ids_archived = []
        min_start_time = None
        max_start_time = None

        # Get the database connection if needed
        if not hasattr(self, "session"):
            self.session = Connector.manager[PIPELINE_DATABASE].session

        # Find entries in file_log with is_delivered and not is_archived
        is_not_archived_filter = FileLog.is_archived == False  # noqa: E712
        is_delivered_filter = FileLog.is_delivered == True  # noqa: E712
        fields = [getattr(FileLog, f) for f in ["id", "basename", "public_filename"]]

        query = None
        try:
            query = (
                self.session.query(FileLog)
                .options(load_only(*fields))
                .filter(and_(is_not_archived_filter, is_delivered_filter))
            )
            results = query.all()
        except Exception as e:
            logger.error("Query failed :")
            if query:
                logger.error(
                    str(
                        query.statement.compile(
                            dialect=postgresql.dialect(),
                            compile_kwargs={"literal_binds": True},
                        )
                    )
                )
            logger.debug(e)
            results = []

        cpt_files_to_archive = len(results)
        logger.info(f"{cpt_files_to_archive} files to store as is_archived")

        # Find min anx max times for these files
        # This will be use to filter the TAP query, in order to limit
        # the data volume to retreive
        try:
            query_min = self.session.query(
                func.min(FileLog.start_time), func.max(FileLog.end_time)
            ).filter(and_(is_not_archived_filter, is_delivered_filter))
            min_start_time = query_min.one()[0]
            max_start_time = query_min.one()[1]
        except Exception:
            logger.error("Query failed :")
            logger.error(
                str(
                    query_min.statement.compile(
                        dialect=postgresql.dialect(),
                        compile_kwargs={"literal_binds": True},
                    )
                )
            )

        if min_start_time is not None:
            min_start_time = min_start_time - timedelta(days=1)
            min_start_time = min_start_time.strftime("%Y-%m-%d")
            logger.debug(f"Filtering on begin_time > {min_start_time}")

        if max_start_time is not None:
            max_start_time = max_start_time + timedelta(days=1)
            max_start_time = max_start_time.strftime("%Y-%m-%d")
            logger.debug(f"Filtering on begin_time < {max_start_time}")

        # Build URL of the query
        select_query = ["SELECT", "file_name", "FROM", "v_public_files"]

        where_query = ["WHERE", "instrument", "=", "%27RPW%27"]
        if min_start_time is not None:
            gt = urllib.parse.quote(">")
            where_query += ["AND", "begin_time", gt, f"%27{min_start_time}%27"]
        if max_start_time is not None:
            lt = urllib.parse.quote("<")
            where_query += ["AND", "begin_time", lt, f"%27{max_start_time}%27"]

        order_query = ["ORDER", "BY", "archived_on", "ASC"]

        tap_query = "+".join(select_query + where_query + order_query)
        query_params = "&".join(
            ["REQUEST=doQuery", "LANG=ADQL", "FORMAT=json", f"QUERY={tap_query}"]
        )
        query_url = f"{self.soar_tap_url}/sync?{query_params}"
        logger.debug(query_url)

        # Run TAP query
        data = None
        try:
            logger.info(f"Querying --> {query_url}")
            data = CheckSOARDataAvailability.soar_run_query(query_url)
        except HTTPError as e:
            logger.error(f"SOAR TAP query has failed ({query_url})")
            logger.error(e)
            logger.exception(e.msg)

        # re-organise data
        archived_data = [d[0] for d in data["data"]]

        for r in results:
            if r.public_filename in archived_data:
                logger.info(f"{r.basename} -> is_archived")
                ids_archived.append(r.id)
            else:
                logger.warning(f"{r.basename} not archived")

        cpt_to_archive = len(ids_archived)

        # update DB
        try:
            id_filter = FileLog.id.in_(ids_archived)
            query = self.session.query(FileLog).filter(id_filter)

            item = {}
            item["is_archived"] = True
            cpt_archived = query.update(item, synchronize_session=False)
        except Exception as e:
            logger.error(f"is_archived update has failed: \n {e}")
            logger.error(
                str(
                    query.statement.compile(
                        dialect=postgresql.dialect(),
                        compile_kwargs={"literal_binds": True},
                    )
                )
            )

        if cpt_archived != cpt_to_archive:
            # this should not happen
            logger.warning(
                f"[{cpt_archived}/{cpt_to_archive}] files stored as is_archived"
            )
        else:
            logger.info(f"{cpt_archived} files stored as is_archived")

        # number of archived files
        self.outputs["store_archived"].data = {
            "files_to_archive": cpt_files_to_archive,
            "files_archived": cpt_archived,
        }

    @staticmethod
    def soar_run_query(query: str) -> dict:
        """
        Runs a ADQL query on SOAR TAP server and return the data table.

        :param query: ADQL query_adql full URL string
        :return: a dictionary containing the query results
        """
        with urllib.request.urlopen(query) as url:
            data = json.loads(url.read().decode())
        return data



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Tasks for handling files processed by DINGO plugin."""

import os
from pathlib import Path

from poppy.core.logger import logger
from poppy.core.task import Task

__all__ = ["MoveProcessedFiles", "MoveFailedFiles"]

from roc.dingo.tools import safe_move


class MoveProcessedFiles(Task):
    """Task to move input list of well processed files in a given target directory."""

    plugin_name = "roc.dingo"
    name = "move_processed_files"

    def setup_inputs(self):
        # See if --no-move keyword is defined
        self.no_move = self.pipeline.get("no_move", default=False, args=True)

        # See if --copy keyword is defined
        self.copy = self.pipeline.get("copy", default=False, args=True)

        # Get or create processed_files list from pipeline properties
        self.processed_files = self.pipeline.get(
            "processed_files", default=[], create=True
        )

        # Get or create processed_files_dir from pipeline properties
        self.processed_files_dir = self.pipeline.get(
            "processed_files_dir", default=[None], create=True
        )[0]

    def run(self):
        try:
            self.setup_inputs()
        except Exception as e:
            logger.error("Initializing inputs for task MoveProcessedFiles has failed!")
            logger.debug(e)
            self.pipeline.exit()
            return

        if self.no_move:
            logger.debug("Skip MoveProcessedFiles task")
            return

        if not self.processed_files:
            logger.debug(
                "Input list of processed files is empty: skip MoveProcessedFiles task"
            )
            return
        else:
            if not self.processed_files_dir:
                logger.debug("processed_files_dir argument not passed, skip task")
                return

            # Create folder if it does not exist
            Path(self.processed_files_dir).mkdir(parents=True, exist_ok=True)

            for current_file in self.processed_files:
                logger.info(f"Moving {current_file} into {self.processed_files_dir}")
                if not safe_move(
                    current_file, self.processed_files_dir, copy=self.copy
                ):
                    logger.error(
                        f"Cannot move {current_file} into {self.processed_files_dir}"
                    )


class MoveFailedFiles(Task):
    """Move failed files found
    into a target directory."""

    plugin_name = "roc.dingo"
    name = "move_failed_files"

    def setup_inputs(self):
        # See if --no-move keyword is defined
        self.no_move = self.pipeline.get("no_move", default=False, args=True)

        # See if --copy keyword is defined
        self.copy = self.pipeline.get("copy", default=False, args=True)

        # Get or create failed_files list from pipeline properties
        self.failed_files = self.pipeline.get("failed_files", default=[], create=True)

        # Get or create failed_files_dir list from pipeline properties
        self.failed_files_dir = self.pipeline.get(
            "failed_files_dir", default=[None], create=True
        )[0]

    def run(self):
        try:
            self.setup_inputs()
        except Exception as e:
            logger.error("Initializing inputs for task MoveFailedFiles has failed!")
            logger.debug(e)
            self.pipeline.exit()
            return

        if self.no_move:
            logger.debug("Skip current task MoveFailedFiles")
            return

        if not self.failed_files:
            logger.debug(
                "Input list of failed files is empty: skip MoveFailedFiles task"
            )
            return
        else:
            if not self.failed_files_dir:
                logger.warning(
                    "There are failed files but failed_files_dir value is not defined, skip MoveFailedFiles task"
                )
                return

            # Create folder if it does not exist
            Path(self.failed_files_dir).mkdir(parents=True, exist_ok=True)

            for current_file in self.failed_files:
                logger.info(f"Moving {current_file} into {self.failed_files_dir}")
                # If failed file does not exist, create an empty file
                if not os.path.isfile(current_file):
                    Path(current_file).touch(exist_ok=True)

                if not safe_move(current_file, self.failed_files_dir, copy=self.copy):
                    logger.error(
                        f"Cannot move {current_file} into {self.failed_files_dir}"
                    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Tests module for the roc.dingo plugin.
"""

import pytest
import os.path as osp

from poppy.core.test import CommandTestCase

from roc.dingo.models.file import FileLog
from roc.dingo.tests.test_log_file import DingoLogFileTest
from roc.dingo.tests.test_dingo import DingoTest


# Tests on roc.dingo.tasks.log_file methods
class TestDingoCheckSoar(CommandTestCase):
    def setup_method(self, method):
        super().setup_method(method)

        # Create the database schemas and tables
        DingoTest.create_db(self)

    def teardown_method(self, method):
        super().teardown_method(method)

        # close and delete database
        self.session.close()
        DingoTest.clear_db(self)

    @pytest.mark.parametrize(
        "root, public_root, delivered_root, nb_archived_files",
        [
            (
                "logfile_to_rocdb/DATA-1",
                "logfile_to_rocdb/PUBLIC-1",
                "logfile_to_rocdb/DELIVERED-1",
                72,
            ),
        ],
    )
    def test_check_soar(self, root, public_root, delivered_root, nb_archived_files):
        """
        Insert dataset with logfile_to_rocdb, store delivered file,
        and after check the SOAR report
        """
        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)
        public_root = osp.join(data_test_path, public_root)
        delivered_root = osp.join(data_test_path, delivered_root)

        # --- initialize the Db with files from DATA-1 ---
        main_command = ["pop", "dingo", "logfile_to_rocdb", "--root", root]

        # --- run the command ---
        self.run_command(main_command)

        # --- insert delivered data ---
        main_command = [
            "pop",
            "dingo",
            "store_delivered",
            "--public",
            public_root,
            "--delivered",
            delivered_root,
        ]

        # --- run the command ---
        self.run_command(main_command)

        # --- check for SOAR data availability ---
        main_command = ["pop", "dingo", "check_soar"]

        # --- run the command ---
        self.run_command(main_command)

        # --- make assertions ---

        # Count the is_archived files
        results = self.session.query(FileLog).filter(FileLog.is_archived == True).all()  # noqa: E712

        assert len(results) == nb_archived_files

        self.session.close()



# -*- coding: utf-8 -*-
"""empty message

Revision ID: roc_dingo_0009_add_bia
Revises: roc_dingo_0008_remove_date
Create Date: 2021-02-09 14:48:04.289978

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "roc_dingo_0009_add_bia"
down_revision = "roc_dingo_0008_remove_date"
branch_labels = None
depends_on = None


def upgrade():
    # ADD BIA in the levels

    op.execute("COMMIT")
    op.execute("ALTER TYPE file_level_list ADD VALUE IF NOT EXISTS 'BIA'")


def downgrade():
    # Postgresl does not offer the ability to remove a value from an
    # enumerate type

    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Contains dingo tasks to synchronize the file tree with the ROC database."""

import os
import re
from csv import DictReader

from sqlalchemy.dialects import postgresql
from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound
from sqlalchemy.orm import load_only

from poppy.core.logger import logger
from poppy.core.db.connector import Connector
from poppy.core.task import Task
from poppy.core.target import BaseTarget

from roc.dingo.constants import (
    PIPELINE_DATABASE,
    DATA_ALLOWED_EXTENSIONS,
    ROC_PUBLIC_DATA_ROOT,
)
from roc.dingo.models.file import FileLog

import h5py
from spacepy.pycdf import CDF


__all__ = ["StorePublicFilename"]


class StorePublicFilename(Task):
    """
    Parse ROC public tree, and for each file get the CDAG_PARENT attribute,
    search for the associated file in the file_log table, and store the
    public filename for this file
    """

    plugin_name = "roc.dingo"
    name = "store_public_filename"

    def add_targets(self):
        self.add_output(
            target_class=BaseTarget,
            many=True,
            identifier="store_public_filename_results",
        )

    def setup_inputs(self):
        """
        Setup task inputs.

        :param task:
        :return:
        """

        logger.debug("StorePublicFilename() : setup_inputs")

        # get the public root file tree
        self.root = self.pipeline.get("public", default=ROC_PUBLIC_DATA_ROOT, args=True)

        # ensure that there is / at the end
        self.root = os.path.join(self.root, "")

    def run(self):
        logger.debug("StorePublicFilename() : run")

        # Initialize inputs
        self.setup_inputs()

        # Get the database connection if needed
        if not hasattr(self, "session"):
            self.session = Connector.manager[PIPELINE_DATABASE].session

        # Get the files already in database (public_* not null)
        fields = [
            getattr(FileLog, f)
            for f in ["id", "basename", "public_filename", "public_dirname"]
        ]
        public_not_null = FileLog.public_filename.isnot(None)

        try:
            query = (
                self.session.query(FileLog)
                .options(load_only(*fields))
                .filter(public_not_null)
            )
            results = query.all()
            logger.info(
                str(
                    query.statement.compile(
                        dialect=postgresql.dialect(),
                        compile_kwargs={"literal_binds": True},
                    )
                )
            )

            files_in_db = [r.public_filename for r in results]
        except Exception:
            logger.error("Query failed :")
            logger.error(
                str(
                    query.statement.compile(
                        dialect=postgresql.dialect(),
                        compile_kwargs={"literal_binds": True},
                    )
                )
            )
            files_in_db = []

        # Levels to be delivered
        levels = ["L0", "L1", "L2", "L3"]

        # Files to be stored
        to_store = []

        # Get the file list
        for level in levels:
            db_file = os.path.join(self.root, level, "db.csv")
            with open(db_file, "r") as db:
                lines = DictReader(db, dialect="unix")
                for line in lines:
                    filepath = line["relative_filepath"]
                    public_filename = os.path.basename(filepath)

                    if public_filename not in files_in_db:
                        public_dirname = os.path.join(level, os.path.dirname(filepath))
                        item = {
                            "public_filename": public_filename,
                            "public_dirname": public_dirname,
                        }
                        to_store.append(item)

        logger.info(f"{len(to_store)} public files to store in DB")

        cpt_to_store = len(to_store)
        cpt_err = 0
        cpt_cur = 0
        cpt_new = 0

        # For each file to be inserted, open it and read the CDAG_PARENT
        # attribute
        for item in to_store:
            cpt_cur += 1
            filepath = os.path.join(
                self.root, item["public_dirname"], item["public_filename"]
            )
            cdag_parent = None
            former_version = "former_versions" in item["public_dirname"]

            attrs_keys = []
            attrs = {}

            logger.info(f"[{cpt_cur}/{cpt_to_store}] Reading {filepath}")
            if ".cdf" in item["public_filename"]:
                try:
                    with CDF(filepath) as fh:
                        attrs_keys = [key for key, val in fh.attrs.items()]
                        attrs = {key: val[...][0] for key, val in fh.attrs.items()}
                except Exception as e:
                    logger.error(f"Error while reading {filepath} : {e}")
                    logger.debug("Break to next")
                    cpt_err += 1
                    continue

            if ".h5" in item["public_filename"]:
                try:
                    with h5py.File(filepath, "r") as fh:
                        attrs_keys = [key for key in fh.attrs.keys()]
                        attrs = {val[0]: val[1] for val in fh.attrs.items()}
                except Exception as e:
                    logger.error(f"Error while reading {filepath} : {e}")
                    logger.debug("Break to next")
                    cpt_err += 1
                    continue

            if "CDAG_PARENT" in attrs_keys:
                cdag_parent = attrs["CDAG_PARENT"]
            elif former_version and "CDAG_PARENTS" in attrs_keys:
                # some old files have CDAG_PARENT*S* attribute
                cdag_parent = attrs["CDAG_PARENTS"]
            else:
                logger.warning(f"No attribute found for {filepath}")
                logger.debug("Break to next")
                cpt_err += 1
                continue

            # search for the cdag parent
            logger.info(f"Parent is {cdag_parent}")
            basename_filter = FileLog.basename == cdag_parent

            file_to_be_added = False

            try:
                query = self.session.query(FileLog).filter(basename_filter)
                result = query.one()
            except MultipleResultsFound:
                logger.warning(f"Multiple files found for {cdag_parent}")
                result = None
            except NoResultFound:
                logger.warning(f"No files found for {cdag_parent}")
                file_to_be_added = True
                result = None

            if result is None and not file_to_be_added:
                cpt_err += 1
                logger.debug("Break to next")
                continue

            # if needed, add the FileLog object in DB, to keep track
            # of all the delivered objects
            if file_to_be_added:
                new_item = {}
                new_item["basename"] = cdag_parent

                # search for version number
                re_ext = "|".join(DATA_ALLOWED_EXTENSIONS)
                res = re.search(r"(v|V)([0-9a-zA-Z]+)\.(" + re_ext + ")", cdag_parent)
                res_id = 2

                if res:
                    new_item["version"] = res.group(res_id)
                else:
                    logger.warning(f"No version number found for {cdag_parent}")
                    cpt_err += 1
                    logger.debug("Break to next")
                    continue

                product = re.sub(r"^([^\.]+)_V[0-9U]+\.[^$]+$", r"\1", cdag_parent)
                new_item["product"] = product

                new_item["state"] = "OK"
                new_item["status"] = "Terminated"
                new_item["creation_time"] = attrs["Generation_date"].strip()
                if new_item["creation_time"] == "":
                    new_item["creation_time"] = None

                new_item["insert_time"] = "now()"
                new_item["dirname"] = item["public_dirname"]
                new_item["dataset_id"] = attrs["Dataset_ID"]
                new_item["level"] = item["public_dirname"].split("/")[0]
                if new_item["level"] not in levels:
                    logger.warning(
                        "Wrong file level found for "
                        f"{cdag_parent} : {new_item['level']}"
                    )
                    cpt_err += 1
                    logger.debug("Break to next")
                    continue

                new_item["is_latest"] = False
                new_item["is_removed"] = True
                # URL is set no NULL because file is removed

                # Add the new FileLog
                new_file = FileLog(**new_item)
                try:
                    self.session.add(new_file)
                    # force flush to be sure to get the insertion error
                    # if needed
                    self.session.flush()
                except Exception as e:
                    logger.error(f"Add of {cdag_parent} has failed: \n {e}")
                    cpt_err += 1
                    continue
                else:
                    cpt_new += 1
                    logger.info(f"{cdag_parent} added successfully")

                # And retreive it
                try:
                    query = self.session.query(FileLog).filter(basename_filter)
                    result = query.one()
                except Exception as e:
                    logger.error(f"Retreive of {cdag_parent} has failed: \n {e}")
                    cpt_err += 1
                    continue

            # update the object
            item_dict = {}
            item_dict["public_filename"] = item["public_filename"]
            item_dict["public_dirname"] = item["public_dirname"]

            try:
                query.update(item_dict)
            except Exception as e:
                logger.error(f"Update of {cdag_parent} has failed: \n {e}")
                cpt_err += 1
            else:
                logger.info(f"{cdag_parent} updated successfully")

        cpt_ok = cpt_to_store - cpt_err
        logger.info(f"{cpt_ok} files have been stored")
        if cpt_err > 0:
            logger.warning(f"{cpt_err} files in error")

        # number of public_filenames stored
        self.outputs["store_public_filename_results"].data = {
            "files_stored": cpt_ok,
            "files_added": cpt_new,
        }



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Tests module for the roc.dingo plugin.
"""

import pytest
import os.path as osp

from poppy.core.test import CommandTestCase

from roc.dingo.models.file import FileLog
from roc.dingo.tests.test_dingo import DingoTest
from roc.dingo.tests.test_log_file import DingoLogFileTest


# Tests on roc.dingo.tasks.log_file methods
class TestDingoLogFileUpdate(CommandTestCase):
    def setup_method(self, method):
        super().setup_method(method)

        # empty the database
        self.session.query(FileLog).delete()
        self.session.flush()

    def teardown_method(self, method):
        super().teardown_method(method)

        # close database
        self.session.close()

    @pytest.mark.parametrize(
        "root, nb_files, file_to_update",
        [
            (
                "logfile_to_rocdb/DATA-1",
                61,
                "solo_L1_rpw-hfr-surv-cdag_20200819_V07.cdf",
            ),
        ],
    )
    def test_logfile_update(self, root, nb_files, file_to_update):
        """
        After data insertion, modify file_to_update and test its update
        """
        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)

        # --- initialize the command ---
        main_command = [
            "pop",
            "-ll",
            "INFO",
            "dingo",
            "logfile_to_rocdb",
            "--root",
            root,
        ]

        # --- run the command ---
        self.run_command(main_command)

        results = self.session.query(FileLog).all()
        assert len(results) == nb_files

        # modify DB to mark a file to be updated
        # and set to null some fields

        db_file_before = DingoTest.get_db_values(
            self.session.query(FileLog).filter(FileLog.basename == file_to_update).one()
        )

        db_file = {
            "to_update": True,
            "creation_time": "1970-01-01",
            "url": "",
            "dataset_id": "",
            "version": "0",
            "start_time": "1970-01-01",
            "end_time": "1970-01-01",
            "product": "",
            "is_delivered": True,
            "is_archived": True,
            "public_filename": "x",
            "public_dirname": "y",
        }

        fields = db_file.keys()
        # these fields will not be changed by the update
        # because they are handled by another task
        not_changed_fields = [
            "is_delivered",
            "is_archived",
            "public_filename",
            "public_dirname",
        ]

        self.session.query(FileLog).filter(FileLog.basename == file_to_update).update(
            db_file
        )
        self.session.flush()

        # -- get the modified version
        db_file_modified = DingoTest.get_db_values(
            self.session.query(FileLog).filter(FileLog.basename == file_to_update).one()
        )

        # --- make assertions ---
        for key in fields:
            assert db_file_modified[key] != db_file_before[key]

        # --- initialize the command ---
        main_command = ["pop", "-ll", "INFO", "dingo", "update_filelog", "--root", root]

        # --- run the command ---
        self.run_command(main_command)

        # -- get the new version
        db_file_after = DingoTest.get_db_values(
            self.session.query(FileLog).filter(FileLog.basename == file_to_update).one()
        )
        #
        # # --- make assertions ---
        for key in fields:
            if key in not_changed_fields:
                # these fields are not changed by update_filelog
                assert db_file_after[key] != db_file_before[key]
                assert db_file_after[key] == db_file_modified[key]
            else:
                # these fields have to get their value back
                assert db_file_after[key] == db_file_before[key]
                assert db_file_after[key] != db_file_modified[key]

        # -- also check that there was one file updated
        # wil be done after Outflow migration

        # self.session.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Contains dingo tasks to insert SOLO/RPW-related event data into the ROC database."""

import uuid
from datetime import datetime, timedelta

import numpy as np
import pandas as pd
from sqlalchemy import and_, or_, null

from poppy.core.logger import logger
from poppy.core.db.connector import Connector
from poppy.core.task import Task

from roc.rpl.time import Time

from roc.dingo.models.data import EventLog, EfecsEvents, BiaSweepLog, SbmLog
from roc.dingo.constants import (
    PIPELINE_DATABASE,
    TRYOUTS,
    TIME_WAIT_SEC,
    SQL_LIMIT,
    TF_PA_DPU_0038,
    TF_PA_DPU_0039,
    IDB_SOURCE,
    EFECS_EVENT_LOG,
    TM_APID_EVENT_LOG,
    TC_PACKET_EVENT_LOG,
    HFR_FREQ_LIST_PACKETS,
    BIA_CURRENT_LOG_PACKETS,
    BIA_SWEEP_TABLE_PACKETS,
    BIA_SWEEP_TABLE_NR,
    LFR_CALIB_EVENT_LOG,
    CIWT0131TM,
)
from roc.dingo.models.packet import TmLog, TcLog

from roc.dingo.tools import (
    load_spice,
    query_db,
    gen_sql_filters,
    get_columns,
    sbm_qf_eng,
    get_current_idb,
    raw_to_na,
    compute_hfr_list_freq,
    insert_on_conflict_update,
)

__all__ = ["EventToDb"]


class EventToDb(Task):
    """
    Insert SOLO/RPW-related event data
    (from database,  SOLO HK data)
    into the ROC database
    """

    plugin_name = "roc.dingo"
    name = "event_to_db"

    def add_targets(self):
        pass

    @Connector.if_connected(PIPELINE_DATABASE)
    def setup_inputs(self):
        # get the input files
        self.start_time = self.pipeline.get("start_time", default=[None])[0]
        self.end_time = self.pipeline.get("end_time", default=[None])[0]

        # Get SOLO SPICE kernels (SCLK and LSK)
        self.sclk_file = self.pipeline.get("sclk", default=[None], create=True)[0]
        self.lsk_file = self.pipeline.get("lsk", default=[None], create=True)[0]
        if not self.sclk_file or not self.lsk_file:
            logger.debug(
                "sclk_file and/or lsk_file not found, try to load SPICE kernels from variable environment."
            )

            # Get (optional) arguments for SPICE
            predictive = self.pipeline.get("predictive", default=False, args=True)
            kernel_date = self.pipeline.get("kernel_date", default=None, args=True)
            no_spice = self.pipeline.get("no_spice", default=False, args=True)

            # Get/create Time singleton
            self.time = Time(
                predictive=predictive, kernel_date=kernel_date, no_spice=no_spice
            )
            self.spice = self.time.spice
        else:
            # Load SPICE kernels
            self.spice = load_spice(spice_kernels=[self.lsk_file, self.sclk_file])

        # Get tryouts from pipeline properties
        self.tryouts = self.pipeline.get("tryouts", default=[TRYOUTS], create=True)[0]

        # Get wait from pipeline properties
        self.wait = self.pipeline.get("wait", default=[TIME_WAIT_SEC], create=True)[0]

        # Retrieve --limit keyword value
        self.limit = self.pipeline.get("limit", default=[SQL_LIMIT], args=True)[0]

        # Get --truncate optional keyword
        self.truncate = self.pipeline.get(
            "truncate",
            default=False,
        )

        # get a database session
        self.session = Connector.manager[PIPELINE_DATABASE].session

        # Get idb_version/idb_source from pipeline properties
        self.idb_source = self.pipeline.get(
            "idb_source", default=[IDB_SOURCE], create=True
        )[0]

        self.idb_version = self.pipeline.get(
            "idb_version", default=[None], create=True
        )[0]

        # If idb_version not passed (is None),
        # then try to get current working version from the database
        if self.idb_version is None:
            self.idb_version = get_current_idb(
                self.idb_source,
                self.session,
                tryouts=self.tryouts,
                wait=self.wait,
            )
        if self.idb_version is None:
            raise ValueError("idb_version argument cannot be defined!")

        # Initialize some class attributes
        self.bia_sweep_tables = None

    def run(self):
        # Define task job ID (long and short)
        self.job_uuid = str(uuid.uuid4())
        self.job_id = self.job_uuid[:8]
        logger.info(f"Task {self.job_id} is starting")
        try:
            self.setup_inputs()
        except Exception as e:
            logger.error(f"Initializing inputs has failed for task {self.job_id}!")
            logger.debug(e)
            self.pipeline.exit()
            return

        if self.truncate:
            event_count = self.session.query(EventLog.id).count()
            logger.info(
                f"Truncating pipeline.{EventLog.__tablename__} table ({event_count} entries found)"
            )
            self.session.execute(f"TRUNCATE TABLE pipeline.{EventLog.__tablename__};")

        # Inserting TM data into event_log table
        logger.debug(
            f"Inserting pipeline.tm_log data into pipeline.event_log\t [{self.job_id}]"
        )
        filters = [TmLog.apid == current_tm for current_tm in TM_APID_EVENT_LOG]
        filters = or_(*filters)
        filters = and_(
            filters,
            gen_sql_filters(TmLog, start_time=self.start_time, end_time=self.end_time),
        )
        tm_inserted, tm_failed, tm_ignored = self.to_event_log(
            TmLog, self.tm_to_event, filters=filters
        )

        # Inserting TC data into event_log table
        logger.debug(
            f"Inserting pipeline.tc_log data into pipeline.event_log\t [{self.job_id}]"
        )
        filters = [
            TcLog.palisade_id == current_tc for current_tc in TC_PACKET_EVENT_LOG
        ]
        filters = or_(*filters)
        filters = and_(
            filters,
            gen_sql_filters(TcLog, start_time=self.start_time, end_time=self.end_time),
        )
        tc_inserted, tc_failed, tc_ignored = self.to_event_log(
            TcLog, self.tc_to_event, filters=filters
        )

        # Inserting E-FECS data into event_log table
        logger.debug(
            f"Inserting pipeline.efecs_events data into pipeline.event_log\t [{self.job_id}]"
        )
        filters = [
            EfecsEvents.name == current_efecs for current_efecs in EFECS_EVENT_LOG
        ]
        filters = or_(*filters)
        filters = and_(
            filters,
            gen_sql_filters(
                EfecsEvents, start_time=self.start_time, end_time=self.end_time
            ),
        )
        efecs_inserted, efecs_failed, efecs_ignored = self.to_event_log(
            EfecsEvents, self.efecs_to_event, filters=filters
        )

        # Inserting SBM data into event_log table
        logger.debug(
            f"Inserting pipeline.sbm_log data into pipeline.event_log\t [{self.job_id}]"
        )
        sbm_inserted, sbm_failed, sbm_ignored = self.to_event_log(
            SbmLog, self.sbm_to_event
        )

        # Inserting TC LFR calibration data into event_log table
        # For LFR calib, get all data since the beginning of the mission
        # (To be sure to not have incomplete calibrations)
        logger.debug(
            f"Inserting LFR calibration TC data into pipeline.event_log\t [{self.job_id}]"
        )
        filters = [
            TcLog.palisade_id == current_tc for current_tc in LFR_CALIB_EVENT_LOG
        ]
        filters = or_(*filters)
        lfrcal_inserted, lfrcal_failed, lfrcal_ignored = self.to_event_log(
            TcLog, self.lfrcalib_to_event, filters=filters
        )

        # Inserting Bias sweep table data into event_log table
        # For sweep tables get all data since the beginning of the mission
        # (To be sure to have the full table loading history)
        filters = [
            TcLog.palisade_id == current_tc for current_tc in BIA_SWEEP_TABLE_PACKETS
        ]
        filters = or_(*filters)
        logger.debug(
            f"Inserting bias sweep table data into pipeline.event_log\t [{self.job_id}]"
        )
        sweeptable_inserted, sweeptable_failed, sweeptable_ignored = self.to_event_log(
            TcLog, self.biasweeptable_to_event, filters=filters
        )

        # Inserting Bias sweep data into event_log table
        # For Bias sweep get all data since the beginning of the mission
        # (To be sure to not have incomplete sweep)
        logger.debug(
            f"Inserting bias sweep data into pipeline.event_log\t [{self.job_id}]"
        )
        sweep_inserted, sweep_failed, sweep_ignored = self.to_event_log(
            BiaSweepLog, self.biasweep_to_event
        )

        inserted = (
            sweep_inserted
            + sweeptable_inserted
            + sbm_inserted
            + efecs_inserted
            + tm_inserted
            + tc_inserted
            + lfrcal_inserted
        )
        failed = (
            sweep_failed
            + sweeptable_failed
            + sbm_failed
            + efecs_failed
            + tm_failed
            + tc_failed
            + lfrcal_failed
        )
        ignored = (
            sweep_ignored
            + sweeptable_ignored
            + sbm_ignored
            + efecs_ignored
            + tm_ignored
            + tc_ignored
            + lfrcal_ignored
        )

        logger.info(f"{inserted} events inserted\t [{self.job_id}]")
        if failed > 0:
            logger.error(f"{failed} events failed\t [{self.job_id}]")
        if ignored > 0:
            logger.warning(f"{ignored} events ignored\t [{self.job_id}]")

    def efecs_to_event(self, data):
        """
        Convert input data rows from pipeline.efecs_events table
        into entries to be saved in pipeline.event_log table.

        :param data: input efecs_events data rows as a pandas.DataFrame object
        :return: pandas.DataFrame object containing Efecs events to be inserted
        """

        # Sub-method to compute end_time from start_time and duration in seconds
        # (Used by pandas.DataFrame.apply() method to get end_time)
        def get_end_time(row):
            return row.utc_time + timedelta(seconds=int(row.attributes["duration"]))

        # Make sure to inserted allowed EFECS events
        efecs_to_insert = pd.DataFrame()
        n_data = data.shape[0]

        # Define output event_log columns values
        efecs_to_insert["label"] = data["name"]
        efecs_to_insert["start_time"] = data["utc_time"]
        efecs_to_insert["is_predictive"] = [True] * n_data

        # Define end_time (if any duration. Othewise end_time=start_time)
        efecs_to_insert["end_time"] = data.apply(
            lambda row: get_end_time(row) if "attributes" in row else row.utc_time,
            axis=1,
        )

        # Set description
        efecs_to_insert["description"] = data["attributes"]

        return efecs_to_insert

    def sbm_to_event(self, data):
        """
        Convert input data rows from pipeline.sbm_log table
        into entries to be saved in pipeline.event_log table.

        :param data: input sbm_log data rows as a pandas.DataFrame object
        :return: a pandas.DataFrame object containing data to be inserted
        """

        # Sub-method to compute sbm event start_time/end_time
        def get_start_end(data):
            # extract SBM algo parameters
            # (stored as a strings)
            params = [int(val) for key, val in data["sbm_algo_param"].items()]
            sbm_time = data["utc_time"]
            if data["sbm_type"] == 1 and len(params) == 3:
                sbm_dt1_sbm1 = params[0]
                sbm_dt2_sbm1 = params[1]
                sbm_dt3_sbm1 = params[2]
                # Compute SBM event start, end and occurred times
                # (see SSS or DAS User manual for details)
                if sbm_dt2_sbm1 < 2 * sbm_dt1_sbm1:
                    end_time = sbm_time + timedelta(seconds=sbm_dt1_sbm1)
                    start_time = end_time - timedelta(seconds=sbm_dt2_sbm1)
                elif sbm_dt2_sbm1 > 2 * sbm_dt1_sbm1:
                    end_time = sbm_time + timedelta(
                        seconds=int(sbm_dt1_sbm1 + sbm_dt3_sbm1)
                    )
                    start_time = end_time - timedelta(seconds=sbm_dt2_sbm1)
                else:
                    end_time = sbm_time + timedelta(seconds=sbm_dt1_sbm1)
                    start_time = end_time - timedelta(seconds=sbm_dt2_sbm1)

                # Set SBM1 quality factor transfer function
                sbm_qf_tf = TF_PA_DPU_0038

            elif data["sbm_type"] == 2 and len(params) == 10:
                sbm_dt = params[0]
                # Compute start_time / end_time
                start_time = sbm_time - timedelta(seconds=(sbm_dt / 2 + 1))
                end_time = sbm_time + timedelta(seconds=(sbm_dt / 2 + 1))

                # Set SBM1 quality factor transfer function
                sbm_qf_tf = TF_PA_DPU_0039
            else:
                logger.error(
                    f"Entry {data} is badly formatted, please check!\t [{self.job_id}]"
                )
                return None, None, None

            # Compute engineering value of quality factor
            sbm_qf = sbm_qf_eng(
                float(data["sbm_qf"]),
                sbm_qf_tf,
                idb_source=self.idb_source,
                idb_version=self.idb_version,
            )

            # store sbm parameters as a JSON format string
            description = {
                key: val
                for key, val in data.items()
                if key in ["cuc_time", "selected", "status"]
            }
            # Add Quality factor
            description["sbm_qf"] = sbm_qf

            # And SBM algo parameters
            for key, val in data["sbm_algo_param"].items():
                description[key] = int(val)

            return start_time, end_time, description

        # initialize output
        sbm_to_insert = pd.DataFrame()

        # Create label and is_predictive
        sbm_to_insert["label"] = data["sbm_type"].apply(lambda row: f"SBM{row} EVENT")
        sbm_to_insert["is_predictive"] = data["utc_time_is_predictive"]

        # Define start_time, end_time and description
        (
            sbm_to_insert["start_time"],
            sbm_to_insert["end_time"],
            sbm_to_insert["description"],
        ) = zip(*data.apply(lambda row: get_start_end(row), axis=1))

        return sbm_to_insert

    def biasweeptable_to_event(self, data):
        """
        Extract BIAS sweep table values from input TC data

        :param data: TC packet data (as a pandas.DataFrame object)
        :return: pandas.DataFrame object containing bias sweep tables to insert
        """

        def to_na(sweep_first_idx, sweep_step_nr, sweep_step_cur, sweep_table=None):
            if sweep_table is None:
                sweep_table = np.empty(BIA_SWEEP_TABLE_NR, dtype=np.float32)
                sweep_table[:] = np.nan
            # Update the sweep table with new current values
            sweep_table[sweep_first_idx : sweep_first_idx + sweep_step_nr] = raw_to_na(
                sweep_step_cur, idb_version=self.idb_version, idb_source=self.idb_source
            )

            # Return current table
            return sweep_table

        # Initialize output
        output_data = []

        # Initialize loop variables
        # We assume the sweep table is empty at the beginning
        sweep_table = np.empty(BIA_SWEEP_TABLE_NR, dtype=np.float32)
        sweep_table[:] = np.nan

        # Loop over each row in the input data
        for current_row in data.to_dict(orient="records"):
            # Get packet name
            packet_name = current_row["palisade_id"]

            # Make sure to start with an empty description
            description = {}
            if packet_name == "TC_DPU_CLEAR_BIAS_SWEEP":
                # if valid clear table command is found, then
                # stores NULL values
                sweep_table[:] = np.nan
                description["sweep_step_na"] = None
                description["sweep_eeprom"] = False
            elif packet_name == "TC_DPU_LOAD_BIAS_SWEEP":
                # If valid load table command is executed,
                # then update the current sweep table values
                sweep_table = to_na(
                    current_row["data"]["CP_DPU_BIA_SWEEP_FIRST_IDX"],
                    current_row["data"]["CP_DPU_BIA_SWEEP_STEP_NR"],
                    current_row["data"]["CP_DPU_BIA_SWEEP_STEP_CUR"],
                    sweep_table=sweep_table,
                )
                # Store sweep table values in description
                description["sweep_step_na"] = list(sweep_table.astype(str))
                # Store EEPROM loading state
                description["sweep_eeprom"] = (
                    int(current_row["data"]["CP_DPU_BIA_SWEEP_EEPROM"]) == 1
                )
            else:
                continue

            # Add a new entry for the event_log table
            new_entry = {
                "label": packet_name,
                "description": description,
                "start_time": current_row["utc_time"],
                "end_time": current_row["utc_time"],
                "is_predictive": True,
            }
            output_data.append(new_entry)

        # Convert list of dictionaries into list of lists
        if len(output_data) > 0:
            # Store Bias sweep table values for bias sweep events (see in
            # self.biasweep_to_event)
            self.bia_sweep_tables = pd.DataFrame.from_records(output_data).sort_values(
                by=["start_time"], ignore_index=True
            )

        return self.bia_sweep_tables

    def biasweep_to_event(self, data):
        """
        Convert input data rows from pipeline.bia_sweep_log table
        into entries to be saved in pipeline.event_log table.

        :param data: input bia_sweep_log data rows (passed as a pandas.DataFrame object)
        :return: a pandas.DataFrame object contains sweeps to insert
        """
        # Initialize output
        output_data = []

        # Loops over the input data to get complete bias sweeps
        sweep_start_buffer = []
        ant_index_buffer = []
        for i, current_row in enumerate(data.to_dict(orient="records")):
            if current_row["sweep_step"] in ["START_ANT1", "START_ANT2", "START_ANT3"]:
                current_ant_index = current_row["sweep_step"][-1]
                if current_ant_index in ant_index_buffer:
                    current_sweep_index = ant_index_buffer.index(current_ant_index)
                    # If antenna index already found in buffer, this is not expected
                    logger.error(
                        f"START_ANT{current_ant_index} already found in antenna index buffer. "
                        f"Bias sweep on {sweep_start_buffer[current_sweep_index]} might be incomplete!\t [{self.job_id}]"
                    )
                    # Remove already existing bia sweep start and antenna index values
                    # from the buffers
                    del sweep_start_buffer[current_sweep_index]
                    del ant_index_buffer[current_sweep_index]

                # Add the antenna index to the antenna index buffer list
                ant_index_buffer.append(current_ant_index)
                # add the sweep start time in sweep_start_buffer list
                sweep_start_buffer.append(current_row["utc_time"])
            elif current_row["sweep_step"] in ["END_ANT1", "END_ANT2", "END_ANT3"]:
                # The END_ANT{ant_index} should match with ones of the previous START_ANT{ant_index} in ant_index_buffer
                # If not returns a warning message and skip sweep
                current_ant_index = current_row["sweep_step"][-1]
                if current_ant_index not in ant_index_buffer:
                    logger.warning(
                        f"{current_row} is not associated with a previous "
                        f"START_ANT{current_ant_index} event, skip sweep\t [{self.job_id}]"
                    )
                    continue
                else:
                    current_sweep_index = ant_index_buffer.index(current_ant_index)
                    current_sweep_start = sweep_start_buffer[current_sweep_index]
                    # Add a new entry for the event_log table
                    new_entry = {
                        "label": f"BIA_SWEEP_ANT{current_ant_index}",
                        "description": null(),
                        "start_time": current_sweep_start,
                        "end_time": current_row["utc_time"],
                        "is_predictive": current_row["utc_time_is_predictive"],
                    }

                    # Retrieve values in nA of the Bias sweep table used for
                    # the current sweep
                    if self.bia_sweep_tables is not None:
                        current_table = self.bia_sweep_tables[
                            self.bia_sweep_tables.start_time <= current_sweep_start
                        ].sort_values(
                            by=["start_time"], ascending=False, ignore_index=True
                        )
                        if (
                            self.bia_sweep_tables.start_time <= current_sweep_start
                        ).any():
                            new_entry["description"] = {
                                "sweep_step_na": current_table.description[0][
                                    "sweep_step_na"
                                ]
                            }
                            output_data.append(new_entry)
                        else:
                            logger.warning(
                                "No Bias sweep table data found "
                                f"for Bias sweep starting on {current_sweep_start}!\t [{self.job_id}]"
                            )
                    else:
                        logger.warning(
                            "No valid Bias sweep table values found for current bias sweep!\t [{self.job_id}]"
                        )

                # Remove bia sweep start and antenna index values
                # from the buffers
                del sweep_start_buffer[current_sweep_index]
                del ant_index_buffer[current_sweep_index]
            elif current_row["sweep_step"] in ["STEP_ANT1", "STEP_ANT2", "STEP_ANT3"]:
                # Ignore detailed bias value steps
                pass
            elif current_row["sweep_step"] in CIWT0131TM.values():
                # If current sweep has ABORTED or MISSING status, then
                # notify it and skip to next sweep
                current_sweep_index = ant_index_buffer[-1]
                current_sweep_start = sweep_start_buffer[-1]
                logger.warning(
                    f"ANT{current_sweep_index} Bias sweep "
                    f"started on {current_sweep_start} "
                    f"is {current_row['sweep_step']}!\t [{self.job_id}]"
                )

                # Restart buffer (assuming there will no other step for the current sweep
                # Next step should be a 'START_ANT1/2/3'
                ant_index_buffer = []
                sweep_start_buffer = []
            else:
                pass

        # Convert list of dictionaries into list of lists
        if len(output_data) > 0:
            # Store bia sweep data as pandas.DataFrame object
            sweep_to_insert = pd.DataFrame.from_records(output_data).sort_values(
                by=["start_time"], ignore_index=True
            )
        else:
            sweep_to_insert = pd.DataFrame()

        return sweep_to_insert

    def tm_to_event(self, data):
        """
        Convert input data rows from pipeline.tm_log table
        into entries to be saved in pipeline.event_log table.

        :param data: input tm_log data rows as a pandas.DataFrame object
        :return: pandas.DataFrame object containing tm_log rows to insert
        """
        # Initialize output
        tm_to_insert = pd.DataFrame()

        tm_to_insert["label"] = data["palisade_id"]
        tm_to_insert["description"] = data["data"]
        tm_to_insert["start_time"] = data["utc_time"]
        tm_to_insert["end_time"] = data["utc_time"]
        tm_to_insert["is_predictive"] = data["utc_time_is_predictive"]

        return tm_to_insert

    def tc_to_event(self, data):
        """
        Convert input data rows from pipeline.tc_log table
        into entries to be saved in pipeline.event_log table.

        :param data: input tc_log data rows passed a pandas.DataFrame object
        :return: pandas.DataFrame object containing tc_log rows to insert
        """
        # Initialize output
        tc_to_insert = pd.DataFrame()

        # Number of input rows
        n_data = data.shape[0]

        tc_to_insert["label"] = data["palisade_id"]
        tc_to_insert["start_time"] = data["utc_time"]
        tc_to_insert["end_time"] = data["utc_time"]
        tc_to_insert["is_predictive"] = [True] * n_data

        # Set description field
        # By default store TC data in the description
        tc_to_insert["description"] = data["data"]
        # For HFR FREQ LIST TC, provide a comprehensive list of frequency values
        # in the description
        where_hfr_list = data["palisade_id"].isin(HFR_FREQ_LIST_PACKETS)
        tc_to_insert.loc[where_hfr_list, ("description")] = data.loc[
            where_hfr_list
        ].apply(
            lambda row: self.extract_hfr_freq_list(
                row["palisade_id"],
                row["data"],
            ),
            axis=1,
        )
        # For Bias current TC, provide a comprehensive value of the bias
        # current in nA
        where_bia_cur = data["palisade_id"].isin(BIA_CURRENT_LOG_PACKETS)
        tc_to_insert.loc[where_bia_cur, ("description")] = data.loc[
            where_bia_cur
        ].apply(
            lambda row: self.extract_bia_current(row["palisade_id"], row["data"]),
            axis=1,
        )

        return tc_to_insert

    def lfrcalib_to_event(self, data):
        """
        Extract LFR calibration events from input TC data

        :param data: TC packet data rows (as a pandas.DataFrame Group)
        :return: pandas.DataFrame object containing the N LFR calibration events
        """

        # Initialize the default value of output
        calib_to_insert = pd.DataFrame()

        # Initialize loop variables
        calib_list = []
        current_calib = None

        # Loop over each row in the input data
        for current_row in data.to_dict(orient="records"):
            # Get packet name
            packet_name = current_row["palisade_id"]

            if packet_name == "TC_LFR_ENABLE_CALIBRATION" and not current_calib:
                # New LFR Calibration has started
                current_calib = current_row
            elif packet_name == "TC_LFR_DISABLE_CALIBRATION" and current_calib:
                # Current LFR calibration has ended

                # Add a new entry for the event_log table
                new_entry = {
                    "label": "LFR CALIBRATION",
                    "description": {"description": "LFR internal calibration"},
                    "start_time": current_calib["utc_time"],
                    "end_time": current_row["utc_time"],
                    "is_predictive": True,
                }
                calib_list.append(new_entry)

                # Reset current calibration event
                current_calib = None
            elif packet_name == "TC_LFR_ENABLE_CALIBRATION" and current_calib:
                logger.warning(
                    f"Incomplete LFR calibration for {current_calib}!\t [{self.job_id}]"
                )
                current_calib = None  # Force new LFR calibration event
            elif packet_name == "TC_LFR_DISABLE_CALIBRATION" and not current_calib:
                logger.warning(
                    "Encounter TC_LFR_DISABLE_CALIBRATION, but not previous TC_LFR_ENABLE_CALIBRATION found!\t [{self.job_id}]"
                )

        # Convert list of dictionaries into pandas.DataFrame
        if len(calib_list) > 0:
            calib_to_insert = pd.DataFrame.from_records(calib_list)

        return calib_to_insert

    def extract_hfr_freq_list(self, packet_name, packet_data):
        """
        Extract the list of HFR frequencies from input TCs
        (for HFR LIST mode)

        :param packet_name: Name of the TC (PALISADE ID)
        :param packet_data: TC data (pandas.DataFrame)
        :return: hfr_freq_list dictionary with freq_list and freq_nr
        """
        hfr_freq_list = {}

        # Get mode and HFR band from packet_name
        # (e.g., TC_THR_LOAD_BURST_PAR_2)
        tc_fields = packet_name.split("_")
        # Get first letter of the current mode (N or B)
        mode = tc_fields[3][0]
        # Get HFR band
        band = int(tc_fields[-1][0]) - 1

        # Get number of frequencies to extract
        hfr_freq_list["freq_nr"] = int(
            packet_data[f"SY_THR_{mode}_SET_HLS{band}_NR_FREQ"]
        )
        # Extract list of frequencies in kHz
        hfr_freq_list["freq_list"] = [
            str(compute_hfr_list_freq(current_freq)).strip()
            for current_freq in packet_data[f"SY_THR_{mode}_SET_HLS_FREQ_HF{band}"]
        ]

        return hfr_freq_list

    def extract_bia_current(self, packet_name, packet_data):
        """
        Extract BIAS current value from input TC packet data

        :param packet_name: Name of the TC packet (palisade ID)
        :param packet_data: bias current TC data as a dictionary
        :return: Bias current value in nA set on current antenna
        """
        # Define CP_BIA_SET_BIAS{i} TC name (where i = [0, 1, 2])
        param_name = "CP_BIA_SET_BIAS{0}".format(packet_name[-1])
        # Get Bias current value in nA for the corresponding sensor
        # (requires IDB for transfer function)
        return {
            param_name: float(
                raw_to_na(
                    packet_data[param_name],
                    idb_source=self.idb_source,
                    idb_version=self.idb_version,
                )
            )
        }

    def to_event_log(self, model, func, filters=None):
        """
        Generic method to insert data into the pipeline.event_log table.

        :param model: table class of data to copy into event_log table
        :param func: method to use to map data between input and output table fields
        :param filters: list of filters for database query (time range filtering will be automatically appended)

        :return: inserted, failed and ignored entry counters + input data returned from database
        """
        # Get data from pipeline table
        data = self.query_data(model, filters=filters)

        n_data = data.shape[0]
        logger.debug(
            f"{n_data} {model.__tablename__} data to insert into event_log "
            f"between {self.start_time} and {self.end_time}\t [{self.job_id}"
        )
        if n_data == 0:
            return 0, 0, 0

        # Convert data into entries compatible with pipeline.event_log table
        data_from_db = func(data)
        n_to_insert = data_from_db.shape[0]

        # Insert data (or update if already exist)
        inserted_count = 0
        failed_count = 0
        ignored_count = 0
        if n_to_insert > 0:
            # Add insertion times
            data_from_db["insert_time"] = [datetime.today()] * n_to_insert

            # Make sure to keep only expected columns to insert
            columns = get_columns(EventLog, remove=["id"])
            # convert to list of dictionaries
            data_to_insert = data_from_db[columns].to_dict("records")

            logger.debug(
                f"Inserting {n_to_insert} data into table {EventLog.__tablename__}"
            )
            try:
                n_inserted = insert_on_conflict_update(
                    EventLog,
                    self.session,
                    data_to_insert,
                    constraint="event_log_start_time_end_time_label_description_key",
                    set_from_excluded=True,
                )
            except Exception as e:
                logger.exception(
                    f"Insertion in table {EventLog.__tablename__} has failed\t[{self.job_id}]"
                )
                logger.debug(e)
                failed_count = n_to_insert
                n_inserted = 0
            else:
                inserted_count = n_inserted

            if n_inserted > 0:
                logger.info(
                    f"{n_inserted} entries inserted "
                    f"in table {EventLog.__tablename__}\t[{self.job_id}]"
                )

            ignored_count = n_to_insert - inserted_count - failed_count

        return inserted_count, failed_count, ignored_count

    def df_to_dict(self, data, model, remove=["id"]):
        """
        Convert input pandas.Dataframe containing input data
        into dictionary.
        Keep only table columns as keywords in the output dictionary.

        :param data: pandas.Dataframe containing input data
        :param model: table class
        :param remove: list of fields to remove
        :return: dictionary
        """
        columns = get_columns(model, remove=remove)
        return data[columns].to_dict("records")

    def query_data(self, model, filters=[]):
        """
        Common method to query data from database and
        to return rows as pandas.Dataframe

        :param model: Table class
        :param filters: List of filters to query
        :return: pandas.Dataframe containing query results
        """

        # If not input filters, then filter at least by time range
        if filters is None:
            filters = gen_sql_filters(
                model, start_time=self.start_time, end_time=self.end_time
            )

        # If possible, sort returned rows by increasing UTC times
        try:
            order_by = model.utc_time
        except AttributeError:
            order_by = None

        results = query_db(
            self.session,
            model,
            filters=filters,
            order_by=order_by,
            tryouts=self.tryouts,
            wait=self.wait,
            limit=self.limit,
        )
        return results



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Tests module for the roc.dingo l0_to_db command and related tasks.
"""

import datetime
import glob
import os
import shlex
import tarfile

import pytest
import unittest.mock as mock
from pathlib import Path

from poppy.core.conf import Settings
from poppy.core.logger import logger
from poppy.core.generic.requests import download_file
from poppy.core.test import CommandTestCase

from roc.dingo.models.packet import TmLog, TcLog
from roc.dingo.models.data import SbmLog, BiaSweepLog, LfrKcoeffDump
from roc.dingo.constants import DINGO_CACHE_DIR
from roc.dingo.tests.test_dingo import DingoTest


class TestDingoL0ToDbCommand(CommandTestCase):
    def setup_method(self, method):
        super().setup_method(method)
        self.session.query(TmLog).delete()
        self.session.query(TcLog).delete()
        self.session.query(SbmLog).delete()
        self.session.query(BiaSweepLog).delete()
        self.session.query(LfrKcoeffDump).delete()
        self.session.flush()

    def teardown_method(self, method):
        super().teardown_method(method)

    @pytest.mark.parametrize(
        "idb_source, idb_version, exclude_tm_apid",
        [
            ("MIB", "20200131", "1212 1228 1244 1260 1204 1220 1236 1252 1269 1300"),
        ],
    )
    def test_l0_to_db(self, idb_source, idb_version, exclude_tm_apid):
        # Command name
        cmd = "l0_to_db"

        # First get test data to insert (i.e. L0 input file(s))
        test_data_tarball = "test_data.tar.gz"

        self.install_dir = os.environ.get("ROC_TEST_DATA_PATH", DINGO_CACHE_DIR)
        test_data_dir = os.path.join(self.install_dir, cmd)
        Path(test_data_dir).mkdir(exist_ok=True, parents=True)
        test_data_path = os.path.join(test_data_dir, test_data_tarball)
        if not os.path.isfile(test_data_path):
            # Build test data tarball URL
            test_data_url = "/".join([DingoTest.base_url, cmd, test_data_tarball])
            # Get username/password
            test_dingo = DingoTest()

            logger.info(f"Downloading {test_data_url}")
            download_file(
                test_data_path,
                test_data_url,
                auth=(test_dingo.username, test_dingo.password),
            )
        else:
            logger.info(f"{test_data_path} already found, skip downloading")

        # Extract tarball
        if tarfile.is_tarfile(test_data_path):
            logger.debug(f"Extracting {test_data_path} ...")
            with tarfile.open(test_data_path, "r:*") as tarball:
                tarball.extractall(path=test_data_dir, filter="fully_trusted")
        else:
            raise tarfile.ReadError(f"{test_data_path} is not a valid tarball!")

        # Get list of input files
        input_dir = os.path.join(test_data_dir, "inputs")
        input_file = list(glob.glob(os.path.join(input_dir, "solo_L0_rpw*.h5")))
        if not input_file:
            raise FileNotFoundError(f"No input L0 file found in {input_dir}!")
        elif len(input_file) > 1:
            raise ValueError("/inputs must contain only one L0 test file!")
        else:
            # Make sure to have only file
            input_file = input_file[0]

        # Check that SPICE kernels are present in ./data/spice_kernels folder
        spice_kernels_dir = DingoTest.get_spice_kernel_dir()
        if not os.path.isdir(spice_kernels_dir):
            raise FileNotFoundError(
                f"No SPICE kernel set found in {spice_kernels_dir}!"
            )
        else:
            # Get latest version of SCLK et LSK SPICE kernels
            sclk = max(
                (Path(spice_kernels_dir) / "sclk").glob("solo_ANC_soc-sclk_*.tsc"),
                key=os.path.getctime,
            )
            lsk = max(
                (Path(spice_kernels_dir) / "lsk").glob("naif*.tls"),
                key=os.path.getctime,
            )

        # Install IDB
        DingoTest.load_idb(self, idb_source, idb_version, install_dir=self.install_dir)

        # initialize the main command
        command_to_test = " ".join(
            [
                "pop",
                "dingo",
                "--idb-source",
                idb_source,
                "--idb-version",
                idb_version,
                "--sclk",
                str(sclk),
                "--lsk",
                str(lsk),
                cmd,
                "-l0",
                input_file,
                "--exclude-tm-apid",
                exclude_tm_apid,
                "-ll",
                "INFO",
            ]
        )

        # define the required plugins
        plugin_list = ["poppy.pop", "roc.dingo"]

        # run the command
        # force the value of the plugin list
        with mock.patch.object(
            Settings,
            "configure",
            autospec=True,
            side_effect=self.mock_configure_settings(
                dictionary={"PLUGINS": plugin_list}
            ),
        ):
            logger.debug(f"Running : {command_to_test}")
            self.run_command(shlex.split(command_to_test))

        # Check that TM data are in the database
        logger.debug("Querying pipeline.tm_log table in database ...")
        rows = self.session.query(TmLog).order_by(TmLog.utc_time).all()

        assert isinstance(rows, list)
        assert len(rows) == 205

        assert rows[0].palisade_id == "TM_DPU_EVENT_LE_DPU_AHB"
        assert rows[0].utc_time == datetime.datetime(2022, 5, 27, 0, 11, 9, 979413)
        assert rows[-1].palisade_id == "TM_TDS_TC_EXE_SUCCESS"
        assert rows[-1].utc_time == datetime.datetime(2022, 5, 28, 0, 2, 0, 619423)

        # Check that TC data are in the database
        logger.debug("Querying pipeline.tc_log table in database ...")
        rows = self.session.query(TcLog).order_by(TcLog.utc_time).all()

        assert isinstance(rows, list)
        assert len(rows) == 19

        assert rows[0].palisade_id == "TC_TDS_DUMP_NORMAL_TSWF"
        assert (
            rows[0].sha
            == "f0025d34ff033fc0337c1ecf5326675a1ae9d410f0a1bb01402f780774a9c9eb"
        )
        assert rows[-1].palisade_id == "TC_TDS_DUMP_NORMAL_TSWF"
        assert (
            rows[-1].sha
            == "842de20f0c4371fe925b9ca338e9399988c6bbd57353731ce8c658489311acfa"
        )

        # Check that SBM data are in the database
        logger.debug("Querying pipeline.sbm_log table in database ...")
        rows = self.session.query(SbmLog).order_by(SbmLog.utc_time).all()

        assert isinstance(rows, list)
        assert len(rows) == 4

        assert rows[0].sbm_type == 1
        assert rows[0].cuc_time == "706939205:9843"
        assert rows[-1].sbm_type == 1
        assert rows[-1].cuc_time == "707010490:58727"

        # Check that Bias sweep data are in the database
        logger.debug("Querying pipeline.bia_sweep_log table in database ...")
        rows = self.session.query(BiaSweepLog).order_by(BiaSweepLog.utc_time).all()

        # Convert first/last row objects to dictionaries (for debugging)
        # first_row = DingoTest.get_db_values_columns_only(rows[0])
        # last_row = DingoTest.get_db_values_columns_only(rows[-1])
        # print(first_row)
        # print(last_row)

        assert isinstance(rows, list)
        assert len(rows) == 6

        assert rows[0].sweep_step == "START_ANT1"
        assert rows[0].cuc_time == "706968074:1672"
        assert rows[-1].sweep_step == "END_ANT3"
        assert rows[-1].cuc_time == "706968266:1450"



"""empty message

Revision ID: roc_dingo_0030_add_packet_acq_time
Revises: roc_dingo_0029_tm_log_partitioning
Create Date: 2024-03-19 16:11:52.588641

"""

from alembic import op
import sqlalchemy as sa

from poppy.pop.alembic.helpers import execute, user


# revision identifiers, used by Alembic.
revision = "roc_dingo_0030_add_packet_acq_time"
down_revision = "roc_dingo_0029_tm_log_partitioning"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "tm_log",
        sa.Column("acq_coarse_time", sa.BIGINT(), nullable=True),
        schema="pipeline",
    )
    op.add_column(
        "tm_log",
        sa.Column("acq_fine_time", sa.INTEGER(), nullable=True),
        schema="pipeline",
    )
    # ### end Alembic commands ###

    # added by hand
    # fix table permissions
    grant_user_seq = """GRANT USAGE, SELECT
        ON ALL SEQUENCES IN SCHEMA pipeline TO {0}""".format(user)
    execute(grant_user_seq)


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("tm_log", "acq_fine_time", schema="pipeline")
    op.drop_column("tm_log", "acq_coarse_time", schema="pipeline")
    # ### end Alembic commands ###



# -*- coding: utf-8 -*-
"""Modify file_log table


Revision ID: roc_dingo_0004_modify_file_log
Revises: roc_dingo_0003_add_cascade
Create Date: 2020-07-28 09:12:14.769013

"""

from pathlib import Path

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

from poppy.pop.alembic.helpers import create_table, drop_table, execute, user

# revision identifiers, used by Alembic.
revision = "roc_dingo_0004_modify_file_log"
# down_revision = 'roc_dingo_0003_add_cascade'
down_revision = "roc_dingo_0001_initial"
depends_on = None


def upgrade():
    # rename columns
    op.alter_column("file_log", "id_file_log", new_column_name="id", schema="pipeline")
    op.alter_column(
        "file_log", "file_author", new_column_name="author", schema="pipeline"
    )
    op.alter_column(
        "file_log", "file_basename", new_column_name="basename", schema="pipeline"
    )
    op.alter_column(
        "file_log",
        "file_creation_date",
        new_column_name="creation_time",
        schema="pipeline",
    )
    op.alter_column(
        "file_log", "file_dataset_id", new_column_name="dataset_id", schema="pipeline"
    )
    op.alter_column("file_log", "file_date", new_column_name="date", schema="pipeline")
    op.alter_column(
        "file_log", "file_descr", new_column_name="descr", schema="pipeline"
    )
    op.alter_column(
        "file_log", "file_dir", new_column_name="dirname", schema="pipeline"
    )
    op.alter_column(
        "file_log", "file_endtime", new_column_name="end_time", schema="pipeline"
    )
    op.alter_column(
        "file_log", "file_insert_date", new_column_name="insert_time", schema="pipeline"
    )
    op.alter_column("file_log", "file_sha", new_column_name="sha", schema="pipeline")
    op.alter_column("file_log", "file_size", new_column_name="size", schema="pipeline")
    op.alter_column(
        "file_log", "file_starttime", new_column_name="start_time", schema="pipeline"
    )
    op.alter_column(
        "file_log", "file_state", new_column_name="state", schema="pipeline"
    )
    op.alter_column(
        "file_log", "file_status", new_column_name="status", schema="pipeline"
    )
    op.alter_column("file_log", "file_url", new_column_name="url", schema="pipeline")
    op.alter_column(
        "file_log", "file_version", new_column_name="version", schema="pipeline"
    )

    op.drop_constraint(
        "file_log_file_basename_key", "file_log", schema="pipeline", type_="unique"
    )
    op.create_unique_constraint(
        "file_log_basename_key", "file_log", ["basename"], schema="pipeline"
    )

    # Very dirty hack because SA does not create
    # ENUM types when add_column, only with create_table
    # https://stackoverflow.com/questions/47206201/
    # how-to-use-enum-with-sqlalchemy-and-alembic
    # after deleting the table, the enum type is kept

    op.create_table(
        "dirty_hack",
        sa.Column("id", sa.BIGINT(), primary_key=True),
        sa.Column(
            "type",
            postgresql.ENUM(
                "CAL",
                "HK",
                "L0",
                "L1",
                "L1R",
                "L2",
                "L3",
                "LL01",
                "TC",
                "TM",
                name="file_type_list",
            ),
        ),
        schema="pipeline",
    )
    op.drop_table("dirty_hack", schema="pipeline")

    op.add_column(
        "file_log",
        sa.Column(
            "type",
            postgresql.ENUM(
                "CAL",
                "HK",
                "L0",
                "L1",
                "L1R",
                "L2",
                "L3",
                "LL01",
                "TC",
                "TM",
                name="file_type_list",
            ),
            autoincrement=False,
            nullable=False,
        ),
        schema="pipeline",
    )
    op.add_column(
        "file_log",
        sa.Column("is_latest", sa.BOOLEAN(), nullable=False),
        schema="pipeline",
    )
    op.add_column(
        "file_log", sa.Column("validity_end", postgresql.TIMESTAMP()), schema="pipeline"
    )
    op.add_column(
        "file_log",
        sa.Column("validity_start", postgresql.TIMESTAMP()),
        schema="pipeline",
    )

    op.drop_column("file_log", "file_name", schema="pipeline")
    op.drop_column("file_log", "is_test", schema="pipeline")
    op.drop_column("file_log", "file_parents", schema="pipeline")
    op.drop_column("file_log", "file_id", schema="pipeline")
    op.drop_column("file_log", "file_level", schema="pipeline")

    # Start and stop time can be null
    op.alter_column("file_log", "start_time", nullable=True, schema="pipeline")
    op.alter_column("file_log", "end_time", nullable=True, schema="pipeline")

    # create parents table
    create_table(
        "parents_file_log",
        sa.Column("id", sa.BIGINT(), nullable=False),
        sa.Column("id_child", sa.BIGINT(), nullable=False),
        sa.Column("id_parent", sa.BIGINT(), nullable=False),
        sa.ForeignKeyConstraint(
            ["id_child"], ["pipeline.file_log.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(
            ["id_parent"], ["pipeline.file_log.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("id_child", "id_parent"),
        schema="pipeline",
    )
    # ### end Alembic commands ###

    # fix table permissions
    grant_user_seq = """GRANT USAGE, SELECT
        ON ALL SEQUENCES IN SCHEMA pipeline TO {0}""".format(user)
    execute(grant_user_seq)


def downgrade():
    # This command works
    filename = "SQL/roc_dingo_0004__downgrade.sql"
    sql_file = Path(__file__).parent / filename
    with open(sql_file, "r") as sql_content:
        sql_cmd = sql_content.read()
        op.execute(sql_cmd)

    # first drop parents_file_log table
    drop_table("parents_file_log", schema="pipeline")

    # Start and stop time cannot be null
    op.alter_column("file_log", "start_time", nullable=False, schema="pipeline")
    op.alter_column("file_log", "end_time", nullable=False, schema="pipeline")

    # rename columns

    op.alter_column("file_log", "id", new_column_name="id_file_log", schema="pipeline")
    op.alter_column(
        "file_log", "author", new_column_name="file_author", schema="pipeline"
    )
    op.alter_column(
        "file_log", "basename", new_column_name="file_basename", schema="pipeline"
    )
    op.alter_column(
        "file_log",
        "creation_time",
        new_column_name="file_creation_date",
        schema="pipeline",
    )
    op.alter_column(
        "file_log", "dataset_id", new_column_name="file_dataset_id", schema="pipeline"
    )
    op.alter_column("file_log", "date", new_column_name="file_date", schema="pipeline")
    op.alter_column(
        "file_log", "descr", new_column_name="file_descr", schema="pipeline"
    )
    op.alter_column(
        "file_log", "dirname", new_column_name="file_dir", schema="pipeline"
    )
    op.alter_column(
        "file_log", "end_time", new_column_name="file_endtime", schema="pipeline"
    )
    op.alter_column(
        "file_log", "insert_time", new_column_name="file_insert_date", schema="pipeline"
    )
    op.alter_column("file_log", "sha", new_column_name="file_sha", schema="pipeline")
    op.alter_column("file_log", "size", new_column_name="file_size", schema="pipeline")
    op.alter_column(
        "file_log", "start_time", new_column_name="file_starttime", schema="pipeline"
    )
    op.alter_column(
        "file_log", "state", new_column_name="file_state", schema="pipeline"
    )
    op.alter_column(
        "file_log", "status", new_column_name="file_status", schema="pipeline"
    )
    op.alter_column("file_log", "url", new_column_name="file_url", schema="pipeline")
    op.alter_column(
        "file_log", "version", new_column_name="file_version", schema="pipeline"
    )

    op.drop_constraint(
        "file_log_basename_key", "file_log", schema="pipeline", type_="unique"
    )
    op.create_unique_constraint(
        "file_log_file_basename_key", "file_log", ["file_basename"], schema="pipeline"
    )

    op.drop_column("file_log", "type", schema="pipeline")
    op.drop_column("file_log", "is_latest", schema="pipeline")
    op.drop_column("file_log", "validity_end", schema="pipeline")
    op.drop_column("file_log", "validity_start", schema="pipeline")

    execute("DROP TYPE IF EXISTS file_type_list")

    op.add_column(
        "file_log",
        sa.Column("file_id", sa.String(length=48), nullable=True),
        schema="pipeline",
    )
    op.add_column(
        "file_log",
        sa.Column("file_level", sa.String(length=8), nullable=True),
        schema="pipeline",
    )
    op.add_column(
        "file_log",
        sa.Column(
            "file_parents", sa.VARCHAR(length=512), autoincrement=False, nullable=True
        ),
        schema="pipeline",
    )
    op.add_column(
        "file_log",
        sa.Column("is_test", sa.BOOLEAN(), autoincrement=False, nullable=True),
        schema="pipeline",
    )
    op.add_column(
        "file_log",
        sa.Column(
            "file_name", sa.VARCHAR(length=512), autoincrement=False, nullable=True
        ),
        schema="pipeline",
    )

    # ### end Alembic commands ###



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Tests module for the roc.dingo l0_to_hfrtimelog command and related tasks.
"""

import glob
import os
import shlex
import tarfile

import unittest.mock as mock
from pathlib import Path

from poppy.core.conf import Settings
from poppy.core.logger import logger
from poppy.core.generic.requests import download_file
from poppy.core.test import CommandTestCase

from roc.dingo.models.data import HfrTimeLog

# Tests on roc.dingo l0_to_hfrtimelog command
from roc.dingo.constants import DINGO_CACHE_DIR
from roc.dingo.tests.test_dingo import DingoTest


class TestDingoL0ToHfrtimelogCommand(CommandTestCase):
    def setup_method(self, method):
        super().setup_method(method)

        self.session.query(HfrTimeLog).delete()
        self.session.flush()

    def teardown_method(self, method):
        super().teardown_method(method)

    def test_l0_to_hfrtimelog(self):
        # Command name
        cmd = "l0_to_hfrtimelog"

        # First get test data to insert (i.e. L0 input file(s))
        test_data_tarball = "test_data.tar.gz"

        self.install_dir = os.environ.get("ROC_TEST_DATA_PATH", DINGO_CACHE_DIR)
        test_data_dir = os.path.join(self.install_dir, cmd)
        Path(test_data_dir).mkdir(exist_ok=True, parents=True)
        test_data_path = os.path.join(test_data_dir, test_data_tarball)
        if not os.path.isfile(test_data_path):
            # Build test data tarball URL
            test_data_url = "/".join([DingoTest.base_url, cmd, test_data_tarball])
            # Get username/password
            test_dingo = DingoTest()

            download_file(
                test_data_path,
                test_data_url,
                auth=(test_dingo.username, test_dingo.password),
            )

        # Extract tarball
        if tarfile.is_tarfile(test_data_path):
            logger.info(f"Extracting {test_data_path} ...")
            with tarfile.open(test_data_path, "r:*") as tarball:
                tarball.extractall(path=test_data_dir, filter="fully_trusted")
        else:
            raise tarfile.ReadError(f"{test_data_path} is not a valid tarball!")

        # Get list of input files
        input_dir = os.path.join(test_data_dir, "inputs")
        input_files = list(glob.glob(os.path.join(input_dir, "solo_L0_rpw*.h5")))
        if not input_files:
            raise FileNotFoundError(f"No input L0 file found in {input_dir}")

        # initialize the main command
        command_to_test = " ".join(
            ["pop", "dingo", cmd, "-l0", " ".join(input_files), "-ll", "INFO"]
        )

        # define the required plugins
        plugin_list = ["poppy.pop", "roc.dingo"]

        # run the command
        # force the value of the plugin list
        with mock.patch.object(
            Settings,
            "configure",
            autospec=True,
            side_effect=self.mock_configure_settings(
                dictionary={"PLUGINS": plugin_list}
            ),
        ):
            self.run_command(shlex.split(command_to_test))

        # Check that test data are in the database
        logger.debug("Querying hfr_time_log table in database ...")
        rows = self.session.query(HfrTimeLog).order_by(HfrTimeLog.acq_time).all()

        assert isinstance(rows, list)
        assert len(rows) == 5

        for row in rows:
            print(row.acq_time)

        # Convert first/last row objects to dictionaries (for debugging)
        # first_row = DingoTest.get_db_values_columns_only(rows[0])
        # last_row = DingoTest.get_db_values_columns_only(rows[-1])
        # print(first_row)
        # print(last_row)

        assert rows[0].coarse_time == 704635836
        assert rows[0].delta_time1["70467979131033"][0] == 2755222091
        assert rows[-1].coarse_time == 704722236
        assert rows[-1].delta_time1["70472414932248"][-1] == 122029054

        # assert False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from poppy.core.logger import logger

__all__ = ["DingoException", "DbInsertionError", "DbQueryError", "PacketInsertionError"]


class DingoException(Exception):
    """Main exception for Dingo plugin"""

    pass


class DbInsertionError(Exception):
    """Exception raised if ROC data insertion has failed."""

    def __init__(self, message, *args, **kwargs):
        super(DbInsertionError, self).__init__(*args, **kwargs)
        logger.error(message)
        self.message = message

    #    logger_level = 'warning'
    #    use_traceback = True

    pass


class DbQueryError(Exception):
    """Exception raised if ROC data query has failed."""

    def __init__(self, message, *args, **kwargs):
        super(DbQueryError, self).__init__(*args, **kwargs)
        logger.error(message)
        self.message = message

    #    logger_level = 'warning'
    #    use_traceback = True

    pass


class PacketInsertionError(Exception):
    """Exception raised if ROC packet insertion has failed."""

    def __init__(self, message, *args, **kwargs):
        super(PacketInsertionError, self).__init__(*args, **kwargs)
        logger.error(message)
        self.message = message

    #    logger_level = 'warning'
    #    use_traceback = True

    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Search for items in file_log to be updated
"""

import os

from sqlalchemy.dialects import postgresql
from sqlalchemy.orm import load_only

from datetime import datetime

from poppy.core.logger import logger
from poppy.core.db.connector import Connector
from poppy.core.task import Task
from poppy.core.target import BaseTarget

from roc.dingo.constants import PIPELINE_DATABASE, ROC_DATA_ROOT
from roc.dingo.models.file import FileLog

__all__ = ["SearchForUpdate"]


class SearchForUpdate(Task):
    """
    Parse ROC file tree and synchronize with the the ROC database
    """

    plugin_name = "roc.dingo"
    name = "search_for_update"
    files_to_check = []
    files_to_insert = []
    files_to_update = []

    def add_targets(self):
        logger.debug("SearchForUpdate() : add_targets")

        self.add_output(
            target_class=BaseTarget, many=True, identifier="roc_data_files_to_update"
        )
        self.add_output(
            target_class=BaseTarget, many=True, identifier="roc_data_files_to_insert"
        )

    def setup_inputs(self):
        """
        Setup task inputs.

        :param task:
        :return:
        """

        logger.debug("SearchForUpdate() : setup_inputs")

        # get the root file tree
        self.root = self.pipeline.get("root", default=ROC_DATA_ROOT, args=True)
        # ensure that there is / at the end
        self.root = os.path.join(self.root, "")

    def run(self):
        logger.debug("SearchForUpdate() : run")

        # Get the database connection if needed
        if not hasattr(self, "session"):
            self.session = Connector.manager[PIPELINE_DATABASE].session

        # Initialize inputs
        self.setup_inputs()

        # Initialize outputs
        self.files_to_update = {}

        # Define filters to check file_log entry to update
        update_filter = FileLog.to_update == True  # noqa: E712

        # Define relevant fields to retrieve
        fields = [
            getattr(FileLog, f)
            for f in ["basename", "dirname", "creation_time", "size"]
        ]

        query = None
        try:
            query = (
                self.session.query(FileLog)
                .options(load_only(*fields))
                .filter(update_filter)
            )
            logger.debug(
                str(
                    query.statement.compile(
                        dialect=postgresql.dialect(),
                        compile_kwargs={"literal_binds": True},
                    )
                )
            )
            results = query.all()
        except Exception as e:  # pragma: no cover
            logger.error("Query failed :")
            if query:
                logger.error(
                    str(
                        query.statement.compile(
                            dialect=postgresql.dialect(),
                            compile_kwargs={"literal_binds": True},
                        )
                    )
                )
            logger.debug(e)
        else:
            for item in results:
                full_name = os.path.join(self.root, item.dirname, item.basename)

                # test if the file is still here
                if os.path.exists(full_name):
                    # do not rely on database creation_time and size
                    size = (os.stat(full_name).st_size,)
                    creation_time = datetime.fromtimestamp(
                        int(os.path.getmtime(full_name)), None
                    )
                else:
                    # it was removed, keep the values in db
                    logger.info(
                        f"File {item.basename} is not anymore "
                        "in the file system -> mark as removed"
                    )
                    item.is_removed = True
                    item.state = "OK"
                    item.error_log = None
                    item.to_update = False
                    self.session.commit()
                    # do not continue with this file
                    continue

                self.files_to_update[item.basename] = {
                    "filepath": os.path.join(self.root, item.dirname, item.basename),
                    "creation_time": creation_time,
                    "size": size,
                }

        # Returns outputs
        self.outputs["roc_data_files_to_update"].data = self.files_to_update
        logger.info("{} files to update".format(str(len(self.files_to_update))))
        if len(self.files_to_update) == 0:
            self.outputs["roc_data_files_to_update"].is_empty = True

        self.outputs["roc_data_files_to_insert"].data = {}
        self.outputs["roc_data_files_to_insert"].is_empty = True



"""empty message

Revision ID: roc_dingo_0017_add_efecsevents
Revises: roc_dingo_0016_add_kcoeffbiacur
Create Date: 2021-10-20 15:24:57.007966

"""

import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

from poppy.pop.alembic.helpers import (
    create_table,
    execute,
    user,
    drop_table,
)

# revision identifiers, used by Alembic.
revision = "roc_dingo_0017_add_efecsevents"
down_revision = "roc_dingo_0016_add_kcoeffbiacur"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    create_table(
        "efecs_events",
        sa.Column("id", sa.BIGINT(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("utc_time", postgresql.TIMESTAMP(), nullable=False),
        sa.Column("attributes", postgresql.JSONB(astext_type=sa.Text()), nullable=True),
        sa.Column("ltp_count", sa.SMALLINT(), nullable=False),
        sa.Column("gen_time", postgresql.TIMESTAMP(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("name", "utc_time"),
        schema="pipeline",
    )
    # ### end Alembic commands ###

    # added by hand
    # fix table permissions
    grant_user_seq = """GRANT USAGE, SELECT
        ON ALL SEQUENCES IN SCHEMA pipeline TO {0}""".format(user)
    execute(grant_user_seq)


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    drop_table("efecs_events", schema="pipeline")
    # ### end Alembic commands ###



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Reference documentation
# ROC-GEN-SYS-NTT-00038-LES_Iss01_Rev02(Mission_Database_Description_Document)

"""
Database model for rpw file processing history tables.
"""

from sqlalchemy import ForeignKey, String, Table, Column
from sqlalchemy.orm import relationship
from sqlalchemy.dialects.postgresql import BIGINT, INTEGER, TIMESTAMP, ENUM, BOOLEAN

from poppy.core.db.base import Base
from poppy.core.db.non_null_column import NonNullColumn

__all__ = [
    "FileLog",
]


# File lists
FILE_STATE_LIST = ["OK", "WARNING", "ERROR"]
FILE_STATUS_LIST = ["Pended", "Terminated", "InProgress", "Locked", "Failed"]
FILE_LEVEL_LIST = [
    "CAL",
    "TC",
    "TM",
    "SK",
    "L0",
    "HK",
    "L1",
    "BIA",
    "L1R",
    "L2",
    "L3",
    "LL01",
]

# file_action_list = ['Process', 'Reprocess', 'Lock', 'Unlock', 'Delete']

file_state_enum = ENUM(*FILE_STATE_LIST, name="file_state_list")
file_status_enum = ENUM(*FILE_STATUS_LIST, name="file_status_list")
file_level_enum = ENUM(*FILE_LEVEL_LIST, name="file_level_list")

ParentsFileLog = Table(
    "parents_file_log",
    Base.metadata,
    Column("id_parent", INTEGER, ForeignKey("pipeline.file_log.id")),
    Column("id_child", INTEGER, ForeignKey("pipeline.file_log.id")),
    schema="pipeline",
)


class FileLog(Base):
    """
    Class representation of the table for file_log table in the ROC database.
    """

    id = NonNullColumn(BIGINT(), primary_key=True)
    sha = NonNullColumn(String(64), nullable=True, descr="SHA256 of the file")
    basename = NonNullColumn(String(512), descr="Basename of the file", unique=True)
    product = NonNullColumn(
        String(512), descr="Basename of the file without extension and version number"
    )
    version = NonNullColumn(String(16), descr="Data version of the file")
    state = NonNullColumn(
        file_state_enum,
        descr=f"State of the file. Possible values are: {FILE_STATE_LIST}",
    )
    status = NonNullColumn(
        file_status_enum,
        descr=f"Status of the file. Possible values are: {FILE_STATUS_LIST}",
    )
    level = NonNullColumn(
        file_level_enum,
        descr=f"Level of the file. Possible values are: {FILE_LEVEL_LIST}",
    )
    creation_time = NonNullColumn(
        TIMESTAMP(),
        nullable=True,
        descr="Local date and time of the file creation",
        comment="",
    )
    insert_time = NonNullColumn(
        TIMESTAMP(),
        descr="Local date and time of the file insertion in the database",
        nullable=True,
    )
    descr = NonNullColumn(String(512), nullable=True, descr="Description of the file")
    author = NonNullColumn(String(512), nullable=True, descr="Author of the file")
    dirname = NonNullColumn(
        String(512),
        nullable=True,
        descr="Relative path of the file directory (from data root base)",
    )
    url = NonNullColumn(
        String(512), nullable=True, descr="URL of the file in the HTTPS server"
    )
    size = NonNullColumn(BIGINT(), nullable=True, descr="Size of the file in kilobytes")
    start_time = NonNullColumn(TIMESTAMP(), nullable=True, descr="Start time of file")
    end_time = NonNullColumn(TIMESTAMP(), nullable=True, descr="End time of file")
    validity_start = NonNullColumn(
        TIMESTAMP(), nullable=True, descr="Start time of validity range (for CAL files)"
    )
    validity_end = NonNullColumn(
        TIMESTAMP(), nullable=True, descr="End time of validity range (for CAL files)"
    )
    dataset_id = NonNullColumn(
        String(512), nullable=True, descr="Dataset ID in the ROC database"
    )
    is_archived = NonNullColumn(
        BOOLEAN, default=False, descr="True if file has been archived at ESAC"
    )
    is_delivered = NonNullColumn(
        BOOLEAN, default=False, descr="True if file has been delivered ESAC"
    )
    public_filename = NonNullColumn(
        String(512),
        descr="Public filename of the file delivered to ESAC",
        nullable=True,
        unique=False,
    )
    public_dirname = NonNullColumn(
        String(512),
        descr="Public dirname of the file delivered to ESAC",
        nullable=True,
        unique=False,
    )
    is_latest = NonNullColumn(
        BOOLEAN, default=False, descr="True if the version is the latest one"
    )
    is_removed = NonNullColumn(
        BOOLEAN,
        default=False,
        descr="True if the file has been removed from the file directory",
    )
    error_log = NonNullColumn(
        String(),
        nullable=True,
        descr="Log message when an error occured during insertion",
    )
    to_update = NonNullColumn(
        BOOLEAN, default=False, descr="True if file has to be updated by DINGO"
    )

    parents = relationship(
        "FileLog",
        secondary=ParentsFileLog,
        primaryjoin=ParentsFileLog.c.id_child == id,
        secondaryjoin=ParentsFileLog.c.id_parent == id,
        backref="children",
        cascade="all, delete",
    )

    __tablename__ = "file_log"
    __table_args__ = {
        "schema": "pipeline",
    }

    def as_dict(self):
        # init the dictionary
        log_file_dict = {c.name: getattr(self, c.name) for c in self.__table__.columns}

        # delete some useless columns
        log_file_dict.pop("id")
        log_file_dict.pop("insert_time")

        # datetime formatting
        fields = [
            "creation_time",
            "start_time",
            "end_time",
            "validity_start",
            "validity_end",
        ]
        for f in fields:
            time = log_file_dict.get(f)
            if time is not None:
                log_file_dict[f] = time.isoformat()

        log_file_dict["parents"] = []
        for p in self.parents:
            log_file_dict["parents"].append(p.basename)

        log_file_dict["parents"].sort()

        return log_file_dict



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Contains dingo tasks to update pipeline.sbm_log.retrieved column
into the ROC database."""

import uuid
from typing import Tuple, List, Type

import pandas as pd
from sqlalchemy import and_, or_, update

from poppy.core.logger import logger
from poppy.core.db.connector import Connector
from poppy.core.task import Task

from roc.dingo.models.data import SbmLog
from roc.dingo.models.packet import TmLog
from roc.dingo.tools import query_db, valid_time, round_up, gen_sql_filters
from roc.dingo.constants import (
    PIPELINE_DATABASE,
    TIME_DAILY_STRFORMAT,
    TRYOUTS,
    TIME_WAIT_SEC,
    SQL_LIMIT,
    SBM_SCI_PKT_LIST,
    SBM_SCI_PKT_MAX_NB,
)

__all__ = ["UpdateSbmRetrieved"]


class UpdateSbmRetrieved(Task):
    """
    Update pipeline.sbm_log.retrieved column
    from information provided in the
    pipeline.tm_log table.
    """

    plugin_name = "roc.dingo"
    name = "update_sbm_retrieved"

    def add_targets(self):
        pass

    @Connector.if_connected(PIPELINE_DATABASE)
    def setup_inputs(self):
        # Get start_time input value
        self.start_time = valid_time(
            self.pipeline.get("start_time", default=[None])[0],
            str_format=TIME_DAILY_STRFORMAT,
        )

        # Get end_time input value
        self.end_time = valid_time(
            self.pipeline.get("end_time", default=[None])[0],
            str_format=TIME_DAILY_STRFORMAT,
        )

        # get a database session
        self.session = Connector.manager[PIPELINE_DATABASE].session

        # Get tryouts from pipeline properties
        self.tryouts = self.pipeline.get("tryouts", default=[TRYOUTS], create=True)[0]

        # Get wait from pipeline properties
        self.wait = self.pipeline.get("wait", default=[TIME_WAIT_SEC], create=True)[0]

        # Retrieve --limit keyword value
        self.limit = self.pipeline.get(
            "limit",
            default=[SQL_LIMIT],
        )[0]

    def run(self):
        # Define task job ID (long and short)
        self.job_uuid = str(uuid.uuid4())
        self.job_id = self.job_uuid[:8]
        logger.info(f"Job {self.job_id} is starting")
        try:
            self.setup_inputs()
        except Exception as e:
            logger.error(f"Initializing inputs has failed for {self.job_id}!")
            logger.debug(e)
            self.pipeline.exit()
            return

        # Query SBM events list from ROC database
        model = SbmLog

        filters = list()
        # Add start_time/end_time filters (if passed)
        if self.start_time:
            filters.append(model.utc_time >= str(self.start_time))
        if self.end_time:
            filters.append(model.utc_time <= str(self.end_time))

        results = query_db(
            self.session,
            model,
            filters=(and_(*filters) if filters else None),
            tryouts=self.tryouts,
            wait=self.wait,
            limit=self.limit,
            order_by=model.utc_time,
            to_dict="records",
        )

        n_rec = len(results)
        logger.info(f"{n_rec} SBM event(s) found in pipeline.sbm_log")
        if n_rec == 0:
            return

        # Define data buffer time range functions
        sbm_trange_func = {1: self.compute_sbm1_trange, 2: self.compute_sbm2_trange}

        # Loop on each SBM event
        for i, current_sbm in enumerate(results):
            current_sbm_type = current_sbm["sbm_type"]
            current_sbm_utc = current_sbm["utc_time"]

            # Compute data buffer time range
            # (returned the coarse part of the on-board time only)
            current_sbm_start, current_sbm_end = sbm_trange_func[current_sbm_type](
                current_sbm
            )

            # Extend time range by N seconds before/after
            # to make sure to retrieve all SBM event packets
            # For the current event
            N = 60
            current_sbm_start -= N
            current_sbm_end += N

            # Retrieve SBM event science packets from pipeline.TmLog table
            current_sbm_tm = self.query_sbm_tm_data(
                current_sbm_type, current_sbm_start, current_sbm_end
            )
            n_pkt = len(current_sbm_tm)
            logger.info(
                f"{n_pkt} SBM{current_sbm_type} packets "
                f"retrieved from pipeline.tm_log for event occurred at "
                f"{current_sbm_utc}\t [{n_rec - i - 1}]"
            )

            # Compute info from SBM science packet data
            # and update sbm_log.retrieved column
            _ = self.update_sbm_log(current_sbm_type, current_sbm_tm, current_sbm["id"])

    @staticmethod
    def compute_sbm1_trange(event_data: dict) -> Tuple[int, int]:
        """
        Compute time range of SBM1 event
        using event CUC time in input sbm_log data.
        Only coarse part is returned.

        :param event_data: dictionary containing SBM1 event data from sbm_log

        :return: tuple (start_time, end_time)
        """
        # sbm1_time = source_data['HK_RPW_S20_SBM1_TIME_D']
        utc_time = event_data["utc_time"]
        cuc_time = event_data["cuc_time"]
        cuc_coarse = int(cuc_time.split(":")[0])
        dt1 = event_data["sbm_algo_param"]["SY_DPU_SBM1_DT1_SBM1_D"]
        dt2 = event_data["sbm_algo_param"]["SY_DPU_SBM1_DT2_SBM1_D"]
        dt3 = event_data["sbm_algo_param"]["SY_DPU_SBM1_DT3_SBM1_D"]

        start_time = cuc_coarse - int(0.5 * dt2) - 1
        end_time = cuc_coarse + int(0.5 * dt2) + 1

        end_time_dt3 = cuc_coarse + round_up(dt1 + dt3)
        if end_time_dt3 > end_time:
            logger.debug(
                f"DT1_SBM1[{dt1}] + DT3_SBM1[{dt3}] > 0.5 * DT2_SBM1[{dt2}] for SBM1 event at {utc_time} [{cuc_time}]"
            )
            end_time = end_time_dt3

        return start_time, end_time

    @staticmethod
    def compute_sbm2_trange(event_data: dict) -> Tuple[int, int]:
        """
        Compute time range of SBM2 event
        using event on-board cuc time in input sbm_log data.
        Only coarse part is returned.

        :param event_data: dictionary containing SBM2 event data from sbm_log

        :return: start_time, end_time
        """
        dt = event_data["sbm_algo_param"]["HK_DPU_SBM2_DT_SBM2"]

        start_time = int(event_data["cuc_time"].split(":")[0])
        end_time = start_time + int(dt)
        # print('sbm2', utc, start_time, end_time, dt)

        return start_time, end_time

    def query_sbm_tm_data(
        self, sbm_type: int, sbm_obt_start: int, sbm_obt_end: int, model=TmLog
    ) -> Type[pd.DataFrame]:
        """
        Query SBM science packet data from pipeline.tm_log table
        providing the SBM type, start and end on-board times (coarse part)

        :param sbm_type: Type of SBM (1 or 2)
        :type sbm_type: int
        :param sbm_obt_start: SBM data buffer start on-board time (obt coarse part)
        :type sbm_obt_start: int
        :param sbm_obt_end: SBM data buffer end on-board time (obt coarse part)
        :type sbm_obt_end: int
        :param model: Database table model to use.
        :type model: SQLAlchemy model class
        :return: data returned by query_db() method
        :rtype: Type[pd.DataFrame]
        """
        # Build query filters
        filters = [
            model.palisade_id == current_tm for current_tm in SBM_SCI_PKT_LIST[sbm_type]
        ]
        filters = or_(*filters)
        filters = and_(
            filters,
            gen_sql_filters(
                TmLog,
                start_time=sbm_obt_start,
                end_time=sbm_obt_end,
                field="acq_coarse_time",
            ),
        )

        # Query database
        return query_db(
            self.session,
            model,
            filters=filters,
            tryouts=self.tryouts,
            wait=self.wait,
            limit=self.limit,
        )

    def update_sbm_log(
        self,
        sbm_type: int,
        sbm_tm_data: pd.DataFrame,
        sbm_log_id: int,
        model=SbmLog,
    ) -> dict:
        """
        Extract event size, number of packets and other info
        from input SBM event data then update sbm_log.retrieved column

        :param sbm_type: type of SBM (can be 1 or 2)
        :type sbm_type: int
        :param sbm_tm_data: data as returned by self.query_sbm_data()
        :type sbm_tm_data: Type[pd.DataFrame]
        :param sbm_log_id: sbm_log id for the current event
        :type sbm_log_id: int
        :param model: database table model instance
        :type model: SQLAlchemy model class
        :return: sbm data info as dictionary
        :rtype: dict
        """
        sbm_info = {}

        # For each SBM science packet type, compute
        # the size in bytes and packet counts
        total_size = 0
        total_count = 0
        total_percent = 0.0
        retrieved_time = []
        for current_packet in SBM_SCI_PKT_LIST[sbm_type]:
            # Initialize dict for current packet
            sbm_info[current_packet] = {"size": 0, "count": 0, "percent": 0.0}

            current_index = sbm_tm_data["palisade_id"] == current_packet
            current_length = sbm_tm_data["length"][current_index]

            sbm_info[current_packet]["count"] = int(current_length.count())
            if current_length.count() == 0:
                continue
            sbm_info[current_packet]["size"] = int(
                sum(current_length) + (7 * sbm_info[current_packet]["count"])
            )
            sbm_info[current_packet]["percent"] = 100.0 * (
                float(sbm_info[current_packet]["count"])
                / float(SBM_SCI_PKT_MAX_NB[sbm_type][current_packet])
            )

            # Compute total size, count and percent
            total_size += int(sbm_info[current_packet]["size"])
            total_count += int(sbm_info[current_packet]["count"])
            total_percent += sbm_info[current_packet]["percent"]

            # For retrieved_time, get the insertion time of the
            # latest SBM science packet in tm_log
            retrieved_time.append(max(sbm_tm_data["insert_time"][current_index]))

        # Get number of expected packets
        n_pkt = len(self.get_sbm_sci_pkt_names(sbm_type, no_comp=True))
        total_percent = total_percent / float(n_pkt)

        # Update retrieved column in pipeline.sbm_log table with
        # this information
        retrieved = {
            "size": total_size,
            "percent": total_percent,
            "info": sbm_info,
        }

        values = {"retrieved": retrieved}
        if retrieved_time:
            retrieved_time = max(retrieved_time)
            values["retrieved_time"] = retrieved_time

        stmt = update(model).values(**values).where(model.id == sbm_log_id)
        self.session.execute(stmt)

        return sbm_info

    @staticmethod
    def get_sbm_sci_pkt_names(sbm_type: int, no_comp: bool = False) -> List[str]:
        """
        Get list of names of
        SBM science packets

        :param sbm_type: Type of SBM (1 or 2)
        :type sbm_type: int
        :param no_comp: remove compressed packets from the list
        :type no_comp: bool
        :return: list of names
        :rtype: List[str]
        """
        if no_comp:
            pkt_list = list(
                set(
                    [
                        current_packet.replace("_C", "")
                        for current_packet in SBM_SCI_PKT_LIST[sbm_type]
                    ]
                )
            )
        else:
            pkt_list = SBM_SCI_PKT_LIST[sbm_type]

        return pkt_list



"""empty message

Revision ID: roc_dingo_0023_add_hfrtimelog
Revises: roc_dingo_0022_add_to_update_flag_to_filelog
Create Date: 2022-02-08 17:22:38.827046

"""

import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

from poppy.pop.alembic.helpers import create_table, execute, user, drop_table

# revision identifiers, used by Alembic.
revision = "roc_dingo_0023_add_hfrtimelog"
down_revision = "roc_dingo_0022_add_to_update_flag_to_filelog"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    create_table(
        "hfr_time_log",
        sa.Column("id", sa.BIGINT(), nullable=False),
        sa.Column("acq_time", postgresql.TIMESTAMP(), nullable=False),
        sa.Column("coarse_time", sa.BIGINT(), nullable=False),
        sa.Column("fine_time", sa.BIGINT(), nullable=False),
        sa.Column("mode", sa.SMALLINT(), nullable=False),
        sa.Column(
            "delta_time1", postgresql.JSONB(astext_type=sa.Text()), nullable=True
        ),
        sa.Column(
            "delta_time2", postgresql.JSONB(astext_type=sa.Text()), nullable=True
        ),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("coarse_time", "fine_time", "mode"),
        schema="pipeline",
    )
    # ### end Alembic commands ###

    # added by hand
    # fix table permissions
    grant_user_seq = """GRANT USAGE, SELECT
        ON ALL SEQUENCES IN SCHEMA pipeline TO {0}""".format(user)
    execute(grant_user_seq)


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    drop_table("hfr_time_log", schema="pipeline")
    # ### end Alembic commands ###



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Tests module for the roc.dingo plugin.
"""

import pytest
import tempfile
import shutil
import os.path as osp
from os import listdir

from poppy.core.test import CommandTestCase

from roc.dingo.models.file import FileLog
from roc.dingo.tests.test_log_file import DingoLogFileTest


# @pytest.mark.skip
# Tests on roc.dingo.tasks.log_file methods
class TestDingoStoreDelivered(CommandTestCase):
    def setup_method(self, method):
        super().setup_method(method)

        # empty the database
        self.session.query(FileLog).delete()
        self.session.flush()

    def teardown_method(self, method):
        super().teardown_method(method)

        # close database
        self.session.close()

    @pytest.mark.parametrize(
        "root, public_root, delivered_root, remove_files, "
        "nb_files_after_logfile_to_rocdb, "
        "nb_files_after_store_delivered, "
        "nb_public_files, nb_delivered_files",
        [
            (
                "logfile_to_rocdb/DATA-1",
                "logfile_to_rocdb/PUBLIC-1",
                "logfile_to_rocdb/DELIVERED-1",
                True,
                61,
                106,
                76,
                72,
            ),
            (
                "logfile_to_rocdb/DATA-1",
                "logfile_to_rocdb/PUBLIC-1",
                "logfile_to_rocdb/DELIVERED-1",
                False,
                61,
                106,
                76,
                72,
            ),
        ],
    )
    def test_store_delivered(
        self,
        root,
        public_root,
        delivered_root,
        remove_files,
        nb_files_after_logfile_to_rocdb,
        nb_files_after_store_delivered,
        nb_public_files,
        nb_delivered_files,
    ):
        """
        Insert dataset with logfile_to_rocdb and store delivered file after
        """
        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)
        public_root = osp.join(data_test_path, public_root)
        delivered_root = osp.join(data_test_path, delivered_root)

        # --- initialize the Db with files from DATA-1 ---
        main_command = [
            "pop",
            "-ll",
            "INFO",
            "dingo",
            "logfile_to_rocdb",
            "--root",
            root,
        ]

        # --- run the command ---
        self.run_command(main_command)

        results = self.session.query(FileLog).all()
        assert len(results) == nb_files_after_logfile_to_rocdb

        # No files should be tagged is_removed
        results = self.session.query(FileLog).filter(FileLog.is_removed == True).all()  # noqa: E712
        assert len(results) == 0

        # As the store_delivered command will delete file,
        # copy the folder to a tmp one

        tmp_delivered_root = tempfile.mkdtemp()
        for f in listdir(delivered_root):
            shutil.copy(osp.join(delivered_root, f), tmp_delivered_root)

        # Check the copy
        results = listdir(tmp_delivered_root)
        assert len(results) == nb_delivered_files

        # --- insert delivered data ---
        main_command = [
            "pop",
            "-ll",
            "INFO",
            "dingo",
            "store_delivered",
            "--public",
            public_root,
            "--delivered",
            tmp_delivered_root,
        ]

        if remove_files:
            main_command += ["--remove_files"]

        # --- run the command ---
        self.run_command(main_command)

        # --- make assertions ---

        # Nb files after store_delivered
        results = self.session.query(FileLog).all()
        assert len(results) == nb_files_after_store_delivered

        # All the files added should be tagged is_removed
        results = self.session.query(FileLog).filter(FileLog.is_removed == True).all()  # noqa: E712
        assert (
            len(results)
            == nb_files_after_store_delivered - nb_files_after_logfile_to_rocdb
        )

        # Check the nb of files with a public file mentioned
        results = (
            self.session.query(FileLog)
            .filter(FileLog.public_filename.isnot(None))
            .all()
        )
        assert len(results) == nb_public_files

        # Check the nb of delivered files
        results = self.session.query(FileLog).filter(FileLog.is_delivered == True).all()  # noqa: E712
        assert len(results) == nb_delivered_files

        # Check all the delivered files have been removed
        # if remove_files has been set
        results = listdir(tmp_delivered_root)
        if remove_files:
            assert len(results) == 0
        else:
            assert len(results) == nb_delivered_files

        # remove temporary directory
        shutil.rmtree(tmp_delivered_root)

        # self.session.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Contains dingo tasks to synchronize the file tree with the ROC database."""

import os
import re
import numpy as np

from datetime import datetime

from poppy.core.logger import logger
from poppy.core.db.connector import Connector
from poppy.core.task import Task
from poppy.core.target import BaseTarget

from roc.dingo.constants import (
    PIPELINE_DATABASE,
    ROC_DATA_ROOT,
    FILE_LEVEL_LIST,
    DATA_ALLOWED_EXTENSIONS,
    SPICE_KERNEL_ALLOWED_EXTENSIONS,
    LEVEL_2_INST,
    LEVEL_3_INST,
    SPICE_KERNEL_TYPES,
    LEVEL_BY_MONTH,
    LEVEL_BY_DAY,
)

__all__ = ["ParseFileTree"]


class ParseFileTree(Task):
    """
    Parse ROC file tree and keep unlogged files
    """

    plugin_name = "roc.dingo"
    name = "parse_file_tree"

    def add_targets(self):
        logger.debug("ParseFileTree() : add_targets")

        self.add_output(target_class=BaseTarget, many=True, identifier="roc_data_files")

        self.add_output(target_class=BaseTarget, many=True, identifier="roc_dir_list")

    def setup_inputs(self):
        """
        Setup task inputs.

        :param task:
        :return:
        """

        logger.debug("ParseFileTree() : setup_inputs")

        # get the root file tree
        self.root = self.pipeline.get("root", default=ROC_DATA_ROOT, args=True)
        # ensure that there is / at the end
        self.root = os.path.join(self.root, "")

        # get the date directory to include
        self.date = self.pipeline.get("date", default=None, args=True)

        # get the level directories to include
        # default = all levels but spice kernels (SK)
        all_levels = FILE_LEVEL_LIST.copy()
        all_levels.remove("SK")
        self.level_list = self.pipeline.get("level", default=all_levels, args=True)

        # be sure that levels are read in a logical way
        # (set by file_level_list)
        # in order to parse L0 before L1, HK before L0, ...
        np_file_level_list = np.array(FILE_LEVEL_LIST)
        self.level_list = np_file_level_list[
            np.array([(i in self.level_list) for i in np_file_level_list])
        ]
        logger.debug(self.level_list)

        # get a database session
        self.db = Connector.manager[PIPELINE_DATABASE]

        # get the former-versions flag
        self.no_former_versions = self.pipeline.get(
            "no_former_versions", default=False, args=True
        )

    @Connector.if_connected(PIPELINE_DATABASE)
    def run(self):
        # Initialize inputs
        self.setup_inputs()

        logger.debug("LogFileToDb() : run")

        # Initialize list of ROC data files to insert into db
        file_list = {}

        logger.info("Root directory: {}".format(self.root))
        logger.debug(
            "Directories to include: {}, ({})".format(
                ", ".join(self.level_list), len(self.level_list)
            )
        )

        level_inst = {}
        for level in FILE_LEVEL_LIST:
            level_inst[level] = [""]
            if level == "L2":
                level_inst[level] = LEVEL_2_INST
            if level == "L3":
                level_inst[level] = LEVEL_3_INST
            if level == "SK":
                level_inst[level] = SPICE_KERNEL_TYPES

        # if a date is specified, is it a day or a month ?
        set_date_month = False
        set_date_day = False
        if self.date:
            if len(self.date.split("/")) == 3:
                set_date_day = True
            elif len(self.date.split("/")) == 2:
                set_date_month = True
        else:
            self.date = ""

        # Set the directories to parse
        dir_list = []

        for file_level in self.level_list:
            if set_date_day and file_level not in LEVEL_BY_DAY:
                logger.info(
                    "Skipping {}, not available on a day basis".format(file_level)
                )
                continue
            if (
                set_date_month
                and file_level not in LEVEL_BY_MONTH
                and file_level not in LEVEL_BY_DAY
            ):
                logger.info(
                    "Skipping {}, not available on a month basis".format(file_level)
                )
                continue

            for inst in level_inst[file_level]:
                if file_level == "SK":
                    dir_list.append(os.path.join(inst))
                else:
                    dir_list.append(os.path.join(file_level, inst, self.date))

        logger.debug(dir_list)

        for dir_item in dir_list:
            # file_level = dir_item.split("/")[0]

            logger.debug("========== %s ==========" % dir_item)
            logger.debug("%s", os.path.join(self.root, dir_item))
            for dirName, subdirList, fileList in os.walk(
                os.path.join(self.root, dir_item)
            ):
                logger.debug("-----------------------------")
                logger.debug("Found directory: %s" % dirName)
                logger.debug("Found subdirList: %s" % subdirList)
                logger.debug("Found fileList: %s" % fileList)
                logger.debug("-----------------------------")

                if self.no_former_versions and "former_versions" in dirName:
                    logger.debug("Skipping %s (-nf specified)" % dirName)
                    continue

                for fname in fileList:
                    # Skipping hidden files
                    if re.match(r"\.", fname):
                        logger.debug("Skipping hidden file %s" % fname)
                        continue

                    # Skipping non allowed extensions
                    file_name, file_extension = os.path.splitext(fname)
                    if (
                        file_extension.replace(".", "") not in DATA_ALLOWED_EXTENSIONS
                        and file_extension.replace(".", "")
                        not in SPICE_KERNEL_ALLOWED_EXTENSIONS
                    ):
                        logger.debug("Skipping non authorized %s" % fname)
                        continue

                    full_name = os.path.join(dirName, fname)
                    logger.debug("\t%s" % full_name)
                    file_list[fname] = {
                        "filepath": full_name,
                        "size": os.stat(full_name).st_size,
                        "creation_time": datetime.fromtimestamp(
                            int(os.path.getmtime(full_name)), None
                        ),
                    }

        logger.info("{} files found in root".format(str(len(file_list))))

        self.outputs["roc_data_files"].data = file_list
        self.outputs["roc_dir_list"].data = dir_list



# -*- coding: utf-8 -*-
"""empty message

Revision ID: roc_dingo_0001_initial
Revises: poppy_pop_0001_initial
Create Date: 2020-02-10 13:59:42.454002

"""

import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

from poppy.pop.alembic.helpers import (
    create_table,
    create_schema,
    execute,
    user,
    drop_table,
)

# revision identifiers, used by Alembic.
revision = "roc_dingo_0001_initial"
down_revision = None
branch_labels = None
depends_on = "poppy_pop_0002_version_num"


def upgrade():
    create_schema("pipeline")  # added by hand

    ## delete the types if they exist
    execute("DROP TYPE IF EXISTS file_state_type")
    execute("DROP TYPE IF EXISTS file_status_type")

    # ### commands auto generated by Alembic - please adjust! ###
    create_table(
        "file_log",
        sa.Column("id_file_log", sa.BIGINT(), nullable=False),
        sa.Column("file_sha", sa.String(length=64), nullable=True),
        sa.Column("file_id", sa.String(length=48), nullable=False),
        sa.Column("file_basename", sa.String(length=512), nullable=False),
        sa.Column("file_version", sa.String(length=16), nullable=False),
        sa.Column(
            "file_state",
            postgresql.ENUM("OK", "WARNING", "ERROR", name="file_state_type"),
            nullable=False,
        ),
        sa.Column(
            "file_status",
            postgresql.ENUM(
                "Pended",
                "Terminated",
                "InProgress",
                "Locked",
                "Failed",
                name="file_status_type",
            ),
            nullable=False,
        ),
        sa.Column("file_creation_date", postgresql.TIMESTAMP(), nullable=True),
        sa.Column("file_insert_date", postgresql.TIMESTAMP(), nullable=True),
        sa.Column("file_descr", sa.String(length=512), nullable=True),
        sa.Column("file_name", sa.String(length=512), nullable=True),
        sa.Column("file_author", sa.String(length=512), nullable=True),
        sa.Column("file_dir", sa.String(length=512), nullable=True),
        sa.Column("file_url", sa.String(length=512), nullable=True),
        sa.Column("file_size", sa.BIGINT(), nullable=True),
        sa.Column("file_level", sa.String(length=8), nullable=True),
        sa.Column("file_date", sa.DATE(), nullable=True),
        sa.Column("file_starttime", postgresql.TIMESTAMP(), nullable=False),
        sa.Column("file_endtime", postgresql.TIMESTAMP(), nullable=False),
        sa.Column("file_dataset_id", sa.String(length=512), nullable=True),
        sa.Column("is_archived", sa.BOOLEAN(), nullable=False),
        sa.Column("is_test", sa.BOOLEAN(), nullable=False),
        sa.Column("file_parents", sa.String(length=512), nullable=True),
        sa.PrimaryKeyConstraint("id_file_log"),
        sa.UniqueConstraint("file_basename"),
        schema="pipeline",
    )
    create_table(
        "packet_log",
        sa.Column("id_packet_log", sa.BIGINT(), nullable=False),
        sa.Column("length", sa.SMALLINT(), nullable=False),
        sa.Column("type", sa.String(length=8), nullable=False),
        sa.Column("category", sa.String(length=512), nullable=False),
        sa.Column("apid", sa.SMALLINT(), nullable=True),
        sa.Column("sync_flag", sa.BOOLEAN(), nullable=True),
        sa.Column("utc_time", postgresql.TIMESTAMP(), nullable=False),
        sa.Column("srdb_id", sa.String(length=16), nullable=False),
        sa.Column("palisade_id", sa.String(length=256), nullable=False),
        sa.Column("binary", sa.String(), nullable=False),
        sa.Column("sha", sa.String(), nullable=True),
        sa.Column("idb_version", sa.String(length=128), nullable=True),
        sa.Column("idb_source", sa.String(length=128), nullable=True),
        sa.Column("creation_time", sa.String(length=1024), nullable=True),
        sa.Column("ack_exe_state", sa.String(length=16), nullable=True),
        sa.Column("ack_acc_state", sa.String(length=16), nullable=True),
        sa.Column("sequence_name", sa.String(length=16), nullable=True),
        sa.Column("unique_id", sa.String(length=256), nullable=True),
        sa.Column("insertion_time", postgresql.TIMESTAMP(), nullable=False),
        sa.PrimaryKeyConstraint("id_packet_log"),
        sa.UniqueConstraint("utc_time", "srdb_id"),
        schema="pipeline",
    )
    create_table(
        "invalid_packet_log",
        sa.Column("id_invalid_packet_log", sa.BIGINT(), nullable=False),
        sa.Column("binary", sa.String(), nullable=False),
        sa.Column("insertion_time", postgresql.TIMESTAMP(), nullable=False),
        sa.Column("apid", sa.SMALLINT(), nullable=True),
        sa.Column("srdb_id", sa.String(length=16), nullable=True),
        sa.Column("sha", sa.String(), nullable=True),
        sa.Column("utc_time", postgresql.TIMESTAMP(), nullable=True),
        sa.Column("idb_version", sa.String(length=128), nullable=True),
        sa.Column("idb_source", sa.String(length=128), nullable=True),
        sa.Column("comment", sa.String(), nullable=True),
        sa.PrimaryKeyConstraint("id_invalid_packet_log"),
        sa.UniqueConstraint("binary"),
        schema="pipeline",
    )
    # ### end Alembic commands ###

    # added by hand
    # fix table permissions
    grant_user_seq = """GRANT USAGE, SELECT
        ON ALL SEQUENCES IN SCHEMA pipeline TO {0}""".format(user)
    execute(grant_user_seq)


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    drop_table("packet_log", schema="pipeline")
    drop_table("file_log", schema="pipeline")
    drop_table("invalid_packet_log", schema="pipeline")
    # ### end Alembic commands ###

    ## delete the types if they exist
    execute("DROP TYPE IF EXISTS file_state_type")
    execute("DROP TYPE IF EXISTS file_status_type")

    # Delete pipeline schema
    execute("DROP SCHEMA pipeline")



"""empty message

Revision ID: roc_dingo_0011_update_packet
Revises: roc_dingo_0010_add_view
Create Date: 2021-07-13 10:34:14.379850

"""

import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

from poppy.pop.alembic.helpers import (
    create_table,
    execute,
    user,
    drop_table,
)

# revision identifiers, used by Alembic.
revision = "roc_dingo_0011_update_packet"
down_revision = "roc_dingo_0010_add_view"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    create_table(
        "tc_log",
        sa.Column("id", sa.BIGINT(), nullable=False),
        sa.Column("length", sa.INTEGER(), nullable=False),
        sa.Column("category", sa.String(length=512), nullable=True),
        sa.Column("apid", sa.INTEGER(), nullable=True),
        sa.Column("utc_time", postgresql.TIMESTAMP(), nullable=False),
        sa.Column("srdb_id", sa.String(length=16), nullable=False),
        sa.Column("palisade_id", sa.String(length=256), nullable=False),
        sa.Column("binary", sa.String(), nullable=True),
        sa.Column("data", postgresql.JSONB(astext_type=sa.Text()), nullable=True),
        sa.Column("sha", sa.String(), nullable=False),
        sa.Column("tc_exe_state", sa.String(length=16), nullable=True),
        sa.Column("tc_acc_state", sa.String(length=16), nullable=True),
        sa.Column("sequence_name", sa.String(length=16), nullable=True),
        sa.Column("unique_id", sa.String(length=256), nullable=True),
        sa.Column("insert_time", postgresql.TIMESTAMP(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("sha"),
        schema="pipeline",
    )
    create_table(
        "tm_log",
        sa.Column("id", sa.BIGINT(), nullable=False),
        sa.Column("length", sa.INTEGER(), nullable=True),
        sa.Column("category", sa.String(length=512), nullable=True),
        sa.Column("apid", sa.INTEGER(), nullable=True),
        sa.Column("sync_flag", sa.BOOLEAN(), nullable=True),
        sa.Column("srdb_id", sa.String(length=16), nullable=True),
        sa.Column("palisade_id", sa.String(length=256), nullable=True),
        sa.Column("binary", sa.String(), nullable=True),
        sa.Column("data", postgresql.JSONB(astext_type=sa.Text()), nullable=True),
        sa.Column("sha", sa.String(), nullable=False),
        sa.Column("sequence_cnt", sa.BIGINT(), nullable=True),
        sa.Column("cuc_time", sa.String(length=1024), nullable=False),
        sa.Column("obt_time", postgresql.TIMESTAMP(), nullable=True),
        sa.Column("utc_time", postgresql.TIMESTAMP(), nullable=True),
        sa.Column("utc_time_is_predictive", sa.BOOLEAN(), nullable=True),
        sa.Column("insert_time", postgresql.TIMESTAMP(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("sha"),
        schema="pipeline",
    )
    # ### end Alembic commands ###
    # added by hand
    # Drop old packet tables
    drop_table("packet_log", schema="pipeline")
    drop_table("invalid_packet_log", schema="pipeline")

    # added by hand
    # fix table permissions
    grant_user_seq = """GRANT USAGE, SELECT
        ON ALL SEQUENCES IN SCHEMA pipeline TO {0}""".format(user)
    execute(grant_user_seq)


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    drop_table("tm_log", schema="pipeline")
    drop_table("tc_log", schema="pipeline")
    # ### end Alembic commands ###



# -*- coding: utf-8 -*-
"""empty message

Revision ID: roc_dingo_0031_add_sbm_subtype
Revises: roc_dingo_0030_add_packet_acq_time
Create Date: 2024-04-25 10:44:55.787939

"""

from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = "roc_dingo_0031_add_sbm_subtype"
down_revision = "roc_dingo_0030_add_packet_acq_time"
branch_labels = None
depends_on = None


def upgrade():
    op.add_column(
        "sbm_log",
        sa.Column("sbm_subtype", sa.String(), nullable=True),
        schema="pipeline",
    )


def downgrade():
    op.drop_column("sbm_log", "sbm_subtype", schema="pipeline")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Tests module for the roc.dingo plugin.
"""

import pytest
import os.path as osp
from random import sample, randrange
from argparse import Namespace

from poppy.core.pipeline import Pipeline
from poppy.core.logger import logger
from poppy.core.test import TaskTestCase

from roc.dingo.models.file import FileLog
from roc.dingo.tests.test_log_file import DingoLogFileTest
from roc.dingo.tasks import SearchForUpdate


# Tests on roc.dingo.tasks.log_file methods
class TestDingoLogFileSearchForUpdate(TaskTestCase):
    @pytest.mark.parametrize(
        "root, data_file",
        [
            ("logfile_to_rocdb/DATA-1/", "roc_data_files_all.dat"),
        ],
    )
    def test_search_files_to_update(self, root, data_file):
        """
        Test the DINGO SearchForUpdate task with standard parameters

        :return:
        """

        logger.info("test_search_for_update() ")

        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)
        args = Namespace(root=root, dry_run=False)

        # --- database setup ---
        session = DingoLogFileTest.setup_session()

        # --- define data test --
        data1 = DingoLogFileTest.open_test_data(data_file, root)

        # mark some files to be updated
        nb_to_update = randrange(len(data1))
        logger.debug("Nb to update = {}".format(nb_to_update))
        to_update = sample(list(data1), nb_to_update)
        logger.debug("To update = {}".format(nb_to_update))

        # populate database
        for item, data in data1.items():
            file_log = DingoLogFileTest.data_to_file_log(data, root)
            if item in to_update:
                file_log.to_update = True
            session.add(file_log)
        session.flush()

        # --- initialise task & pipeline --
        self.task = SearchForUpdate()
        self.task.pipeline = Pipeline(args)
        self.task.instantiate_targets()
        self.task.session = session

        # --- run the task ---
        self.run_task()

        # --- get results ---
        roc_files_update = self.task.outputs["roc_data_files_to_update"].data
        roc_files_insert = self.task.outputs["roc_data_files_to_insert"].data

        # --- make assertions ---

        # in any case roc_files_insert should be empty
        assert roc_files_insert == {}

        # compare results with expected data
        assert len(roc_files_update) == nb_to_update
        for item in roc_files_update:
            assert item in to_update

        session.close()

    @pytest.mark.parametrize(
        "root, data_file, missing_file_data",
        [
            (
                "logfile_to_rocdb/DATA-1/",
                "roc_data_files_all.dat",
                "roc_data_old_removed_file.dat",
            ),
        ],
    )
    def test_with_missing_file(self, root, data_file, missing_file_data):
        """
        Test the DINGO SearchForUpdate task with a missing file

        :return:
        """

        logger.info("test_search_for_update() ")

        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)
        args = Namespace(root=root, dry_run=False)

        # --- database setup ---
        session = DingoLogFileTest.setup_session()

        # --- define data test --
        data1 = DingoLogFileTest.open_test_data(data_file, root)

        # populate database
        for item, data in data1.items():
            file_log = DingoLogFileTest.data_to_file_log(data, root)
            session.add(file_log)
        session.flush()

        # add a file in DB that will miss in FS
        # read the object and insert in DB
        # it has to_update = true and is not in FS
        data2 = DingoLogFileTest.open_test_data(missing_file_data, root)
        old_file_name = list(data2)[0]
        item = data2[old_file_name]

        old_file = DingoLogFileTest.data_to_file_log(item, root)
        old_file.to_update = True

        assert not old_file.is_removed

        session.add(old_file)
        session.flush()

        # --- initialise task & pipeline --
        self.task = SearchForUpdate()
        self.task.pipeline = Pipeline(args)
        self.task.instantiate_targets()
        self.task.session = session

        # --- run the task ---
        self.run_task()

        # --- get results ---
        roc_files_update = self.task.outputs["roc_data_files_to_update"].data
        roc_files_insert = self.task.outputs["roc_data_files_to_insert"].data

        # --- make assertions ---

        # both case roc_files_insert and roc_files_update should be empty
        assert roc_files_insert == {}
        assert roc_files_update == {}

        # also check that is has been flagged as removed
        new_item = (
            session.query(FileLog).filter(FileLog.basename == old_file_name).one()
        )

        assert new_item.is_removed
        assert new_item.state == "OK"
        assert new_item.error_log is None
        assert not new_item.to_update

        session.close()

    @pytest.mark.parametrize(
        "root, data_file",
        [
            ("logfile_to_rocdb/DATA-1/", "roc_data_files_all.dat"),
        ],
    )
    def test_with_no_update(self, root, data_file):
        """
        Test the DINGO SearchForUpdate task with no files to update

        :return:
        """

        logger.info("test_search_for_update() ")

        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)
        args = Namespace(root=root, dry_run=False)

        # --- database setup ---
        session = DingoLogFileTest.setup_session()

        # --- define data test --
        data1 = DingoLogFileTest.open_test_data(data_file, root)

        # populate database
        for item, data in data1.items():
            file_log = DingoLogFileTest.data_to_file_log(data, root)
            session.add(file_log)
        session.flush()

        # --- initialise task & pipeline --
        self.task = SearchForUpdate()
        self.task.pipeline = Pipeline(args)
        self.task.instantiate_targets()
        self.task.session = session

        # --- run the task ---
        self.run_task()

        # --- get results ---
        roc_files_update = self.task.outputs["roc_data_files_to_update"]
        roc_files_insert = self.task.outputs["roc_data_files_to_insert"]

        # --- make assertions ---

        # in any case roc_files_insert should be empty
        assert roc_files_insert.data == {}
        assert roc_files_update.data == {}

        assert roc_files_insert.is_empty is True
        assert roc_files_update.is_empty is True

        session.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Tests module for the roc.dingo plugin.
"""

import pytest
import os.path as osp
from argparse import Namespace

from poppy.core.pipeline import Pipeline
from poppy.core.logger import logger
from poppy.core.test import TaskTestCase

from roc.dingo.tests.test_log_file import DingoLogFileTest


# Tests on roc.dingo.tasks methods
class TestDingoLogFileParseFileTree(TaskTestCase):
    @pytest.mark.parametrize(
        "root, dir_file, expected_data_file",
        [
            (
                "logfile_to_rocdb/DATA-1/",
                "roc_dir_list_all.dat",
                "roc_data_files_all.dat",
            ),
        ],
    )
    def test_parse_file_tree(self, root, dir_file, expected_data_file):
        """
        Test the DINGO ParseFileTree task with standard parameters

        :return:
        """

        # --- initialize the task ---
        from roc.dingo.tasks import ParseFileTree

        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)
        args = Namespace(root=root, dry_run=False)

        # --- initialise task & pipeline --
        self.task = ParseFileTree()
        self.task.pipeline = Pipeline(args)

        # --- run the task ---
        self.task.instantiate_targets()
        self.run_task()

        # --- get results ---
        roc_dir_list = self.task.outputs["roc_dir_list"].data
        roc_data_files = self.task.outputs["roc_data_files"].data

        # --- make assertions ---
        # --- DIR LIST ---

        # open the dir_list file with data for test
        roc_dir_list_all = DingoLogFileTest.read_data(dir_file)

        # check content
        assert roc_dir_list == roc_dir_list_all, "dir list is not the one expected"

        # --- FILE LIST ---
        # read expected data
        expected_data = DingoLogFileTest.open_test_data(expected_data_file, root)

        # compare results with expected data
        DingoLogFileTest.compare_data_to_expected(
            expected_data, roc_data_files, ignore=["creation_time"]
        )

    @pytest.mark.parametrize(
        "root, date, dir_file, expected_data_file",
        [
            (
                "logfile_to_rocdb/DATA-1/",
                "2020/08/19",
                "roc_dir_list_2020_08_19.dat",
                "roc_data_files_2020_08_19.dat",
            ),
        ],
    )
    def test_parse_file_tree_with_day_date(
        self, root, date, dir_file, expected_data_file
    ):
        """
        Test the DINGO ParseFileTree task with a day date parameter
        All files set in a month or mission basis should be excluded
        :return:
        """

        # --- initialize the task ---
        from roc.dingo.tasks import ParseFileTree

        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)
        args = Namespace(root=root, date=date, dry_run=False)

        # --- initialise task & pipeline --
        self.task = ParseFileTree()
        self.task.pipeline = Pipeline(args)

        # --- run the task ---
        self.task.instantiate_targets()
        self.run_task()

        # --- get results ---
        roc_dir_list = self.task.outputs["roc_dir_list"].data
        roc_data_files = self.task.outputs["roc_data_files"].data

        logger.debug("--- results ---")
        logger.debug(len(roc_data_files))
        for key, item in roc_data_files.items():
            logger.debug(item["filepath"].replace(root, ""))

        # --- make assertions ---
        # --- DIR LIST ---

        # open the dir_list file with data for test
        roc_dir_list_all = DingoLogFileTest.read_data(dir_file)

        # check content
        assert roc_dir_list == roc_dir_list_all, "dir list is not the one expected"

        # --- FILE LIST ---
        # read expected data
        expected_data = DingoLogFileTest.open_test_data(expected_data_file, root)

        # compare resulsts with expected data
        DingoLogFileTest.compare_data_to_expected(
            expected_data, roc_data_files, ignore=["creation_time"]
        )

    @pytest.mark.parametrize(
        "root, date, dir_file",
        [
            ("logfile_to_rocdb/DATA-1/", "2020/08/18", "roc_dir_list_2020_08_18.dat"),
        ],
    )
    def test_parse_file_tree_with_another_day_date(self, root, date, dir_file):
        """
        Test the DINGO ParseFileTree task with another day date parameter
        File list should be empty
        :return:
        """

        # --- initialize the task ---
        from roc.dingo.tasks import ParseFileTree

        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)
        args = Namespace(root=root, date=date, dry_run=False)

        # --- initialise task & pipeline --
        self.task = ParseFileTree()
        self.task.pipeline = Pipeline(args)

        # --- run the task ---
        self.task.instantiate_targets()
        self.run_task()

        # --- get results ---
        roc_dir_list = self.task.outputs["roc_dir_list"].data
        roc_data_files = self.task.outputs["roc_data_files"].data

        logger.debug("--- results ---")
        logger.debug(len(roc_data_files))
        for key, item in roc_data_files.items():
            logger.debug(item["filepath"].replace(root, ""))

        # --- make assertions ---
        # --- DIR LIST ---

        # open the dir_list file with data for test
        roc_dir_list_all = DingoLogFileTest.read_data(dir_file)

        # check content
        assert roc_dir_list == roc_dir_list_all, "dir list is not the one expected"

        # --- FILE LIST ---
        # read expected data
        expected_data = {}

        # compare resulsts with expected data
        DingoLogFileTest.compare_data_to_expected(
            expected_data, roc_data_files, ignore=["creation_time"]
        )

    @pytest.mark.parametrize(
        "root, date, dir_file, expected_data_file",
        [
            (
                "logfile_to_rocdb/DATA-1/",
                "2020/08",
                "roc_dir_list_2020_08.dat",
                "roc_data_files_2020_08.dat",
            ),
        ],
    )
    def test_parse_file_tree_with_month_date(
        self, root, date, dir_file, expected_data_file
    ):
        """
        Test the DINGO ParseFileTree task with a month date parameter
        All files set in the mission basis should be excluded
        :return:
        """

        # --- initialize the task ---
        from roc.dingo.tasks import ParseFileTree

        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)
        args = Namespace(root=root, date=date, dry_run=False)

        # --- initialise task & pipeline --
        self.task = ParseFileTree()
        self.task.pipeline = Pipeline(args)

        # --- run the task ---
        self.task.instantiate_targets()
        self.run_task()

        # --- get results ---
        roc_dir_list = self.task.outputs["roc_dir_list"].data
        roc_data_files = self.task.outputs["roc_data_files"].data

        logger.debug("--- results ---")
        logger.debug(len(roc_data_files))
        for key, item in roc_data_files.items():
            logger.debug(item["filepath"].replace(root, ""))

        # --- make assertions ---
        # --- DIR LIST ---

        # open the dir_list file with data for test
        roc_dir_list_all = DingoLogFileTest.read_data(dir_file)

        # check content
        assert roc_dir_list == roc_dir_list_all, "dir list is not the one expected"

        # --- FILE LIST ---
        # read expected data
        expected_data = DingoLogFileTest.open_test_data(expected_data_file, root)

        # compare results with expected data
        DingoLogFileTest.compare_data_to_expected(
            expected_data, roc_data_files, ignore=["creation_time"]
        )

    @pytest.mark.parametrize(
        "root, expected_data_file",
        [("logfile_to_rocdb/DATA-1/", "roc_data_files_all.dat")],
    )
    def test_parse_file_tree_nf(self, root, expected_data_file):
        """
        Test the DINGO ParseFileTree task with the option -nf
        files from "former_versions" should be excluded

        :return:
        """

        # --- initialize the task ---
        from roc.dingo.tasks import ParseFileTree

        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)
        args = Namespace(root=root, no_former_versions=True, dry_run=False)

        # --- initialise task & pipeline --
        self.task = ParseFileTree()
        self.task.pipeline = Pipeline(args)

        # --- run the task ---
        self.task.instantiate_targets()
        self.run_task()

        # --- get results ---
        roc_data_files = self.task.outputs["roc_data_files"].data

        # --- make assertions ---
        # useless to test directory here
        # --- FILE LIST ---
        expected_data = DingoLogFileTest.open_test_data(expected_data_file, root)
        # remove "former_versions" items
        for key, item in expected_data.copy().items():
            if "former_versions" in item["filepath"]:
                expected_data.pop(key)

        # compare resulsts with expected data
        DingoLogFileTest.compare_data_to_expected(
            expected_data, roc_data_files, ignore=["creation_time"]
        )

    @pytest.mark.parametrize(
        "root, expected_data_file",
        [
            ("logfile_to_rocdb/SK-1/", "roc_sk_files_1.dat"),
        ],
    )
    def test_parse_file_tree_sk(self, root, expected_data_file):
        """
        Test the DINGO ParseFileTree task for spice kernels

        :return:
        """

        # --- initialize the task ---
        from roc.dingo.tasks import ParseFileTree

        # --- set up arguments ---
        data_test_path = DingoLogFileTest.get_test_data_path()
        root = osp.join(data_test_path, root)
        args = Namespace(root=root, level="SK", dry_run=False)

        # --- initialise task & pipeline --
        self.task = ParseFileTree()
        self.task.pipeline = Pipeline(args)

        # --- run the task ---
        self.task.instantiate_targets()
        self.run_task()

        # --- get results ---
        roc_dir_list = self.task.outputs["roc_dir_list"].data
        roc_data_files = self.task.outputs["roc_data_files"].data

        # --- make assertions
        # --- DIR LIST ---
        roc_dir_list_all = ["ck", "fk", "ik", "lsk", "pck", "sclk", "spk"]
        assert roc_dir_list == roc_dir_list_all, "dir list is not the one expected"

        # --- FILES LIST ---
        # read expected data
        expected_data = DingoLogFileTest.open_test_data(expected_data_file, root)

        # compare results with expected data
        DingoLogFileTest.compare_data_to_expected(
            expected_data, roc_data_files, ignore=["creation_time"]
        )



# -*- coding: utf-8 -*-
"""empty message

Revision ID: roc_dingo_0007_add_error_log
Revises: roc_dingo_0006_add_is_removed
Create Date: 2020-12-17 11:38:09.315512

"""

from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = "roc_dingo_0007_add_error_log"
down_revision = "roc_dingo_0006_add_is_removed"
branch_labels = None
depends_on = None


def upgrade():
    # ADD error_log column

    op.add_column(
        "file_log",
        sa.Column("error_log", sa.String(), nullable=True),
        schema="pipeline",
    )

    # ADD SPICE_KERNEL in the levels

    op.execute("COMMIT")
    op.execute("ALTER TYPE file_level_list ADD VALUE IF NOT EXISTS 'SK'")


def downgrade():
    op.drop_column("file_log", "error_log", schema="pipeline")

    # Postgresl does not offer the ability to remove a value from an
    # enumerate type
    # Sk is not removed from enumerate type
    # the 'IF EXISTS' option is set in upgrade() to allow
    # multiple succesive upgrade/downgrade
